/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2007-2024 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/
#include "abouthelp.h"

AboutHelp::AboutHelp(QDialog* parent)
			: QDialog(parent)
{
	uih.setupUi(&wHelp);
	connect(uih.pB_Close, SIGNAL(clicked()), this, SLOT(dlgClose()));
}

AboutHelp::~AboutHelp()
{
}

void AboutHelp::dlgShow(int iD)						// show dialog: 2=about, 3=Help
{
//cout << "dlgShow reached : " << endl;
	iDlg = iD;
	if (wHelp.isVisible() == true) {
		wHelp.hide();
	}
	textSet();
	if (iDlg == 2) {									// about
		uih.tb_Help->setHtml(sText);
		wHelp.setWindowTitle(tr("about filmat11"));
		wHelp.resize(610, 210);
	} else if (iDlg == 3) {								// help
		uih.tb_Help->setHtml(sText);
		wHelp.setWindowTitle(tr("filmat11 - help"));
		wHelp.showFullScreen();							// maximized?
	} else {
		return;
	}
	
	if (iDlg == 2) {									// 2=about
		qPos = QCursor::pos();
		iD = qPos.x() - 450;
		if (iD < 0) {
			iD = 0;
		}
		iD2 = qPos.y() - 175;
		if (iD2 < 0) {
			iD2 = 0;
		}
		wHelp.move(iD, iD2);
	}
	wHelp.show();
/*
if (wHelp.hasFocus() == true ) {
cout << "help dlgShow : we seem to have focus" << endl;
} else {
cout << "help dlgShow : we do not appear to have focus" << endl;
}

if (wHelp.isActiveWindow() == true ) {
cout << "help dlgShow : we are the active window" << endl;
} else {
cout << "helpdlgShow : we do not appear to be the active window" << endl;
}
*/
}

void AboutHelp::dlgClose()
{
	wHelp.hide();
	emit sigWahClose();
}

void AboutHelp::textSet()						// set text to show: 1=About (html), 2=main menu help (html)
{
	switch (iDlg) {
		case 2:										// about text (html)
			sText = tr("<html><body>"
			    "filmat11 - a video player using mpv, based on Qt6.<br>"
				"Version 0.9.8<br>"
				"(C) 2010-2024 by Helge Kortendieck, alan@kasmithee.de<p>"
				"This program is released under the MIT license."
				"</body></html>");
			break;
		case 3:										// program main help text (html)
			sText = tr("<html><body>"
				"<a name=""Top""><b>Table of content</b></a><br>"
				"<a href=""#Intro"">Introduction</a><br>"
				"<a href=""#Start"">Starting</a><br>"
				"<a href=""#Mpv"">mpv configuration</a><br>"
				"<a href=""#Tags"">Tag handling</a><br>"
				"<a href=""#Options"">Command line options</a><br>"
				"<a href=""#Faq"">FAQ</a><br>"
				"<a href=""#Credits"">Credits</a><p>"

				"<hr>"
				"<a name=""Intro""><b><u>Introduction</u></b></a><p>"
				"filmat11 is a simple video player intended to be used with a big TV screen.<p>"
				"It assumes, that all movies are stored in one root directory.<br>"
				"It uses mpv as a player. mpv is called externally. While playing, the input focus is on mpv. When mpv ends or is closed, the input focus falls back to filmat11.<p>"
				"This should currently work for *.avi, *.mkv, *.mp4, *.mpeg, *.mpg, *.ts, *.vdr and *.webm files.<br>"
				"DVDs or Bluerays are not supported.<p>"
				"It is possible to add tags to a movie and filter the shown list by them. But only 1 filter is allowed at a time. See the <a href=""#Tags"">tags</a> section for more instructions.<p>"
				"<a href=""#Top"">back to top</a><p>"

				"<hr>"
				"<a name=""Start""><b><u>Starting</u></b></a><p>"
				"Start filmat11 by calling just this name as a command. However, on a desktop supporting freedesktop.org, a menu entry should have been "
				"added in the Multimedia or Video sub-menu.<p>"
				"If you start the program for the first time, set the root directory with the movies under settings.<br>"
				"The movie list will then be populated. Start a movie by double clicking on it.<br>"
				"mpv will start by using it's own configuration file, e. g. ~/.config/mpv/mpv.conf . To close mpv hit 'q'. You should come back to filmat11.<p>"
				"<a href=""#Top"">back to top</a><p>"
                  
				"<hr>"
				"<a name=""Mpv""><b><u>mpv configuration</u></b></a><p>"
				"mpv will start by using it's own configuration file, e. g. ~/.config/mpv/mpv.conf .<br>"
				"As a small configuration example, this file might look like this:<br>"
				"fullscreen=yes<br>"
				"hwdec=auto-safe<p>"
				"This will start mpv in fullscreen mode and tries to enable hardware accelleration.<br>"
				"More information can be optained by executing 'man mpv' in a terminal. As already said, you can close mpv by hitting 'q'.<p>"
				"<a href=""#Top"">back to top</a><p>"
				
				"<hr>"
				"<a name=""Tags""><b><u>Tag handling</u></b></a><p>"
				"You can add some tags to each movie and filter the movie list by 1 of them.<p>"
				"Right-click on a movie entry to open the tag editor.<br>"
				"When entering a tag, you can select an entry from already existing tags, which might come handy to avoid cluttering the tag list with accidental typos.<br>"
				"When using a remote keyboard, you can use TAB to faster navigate from tag to tag entry and SPACE to 'press' an Cancel-, OK- or SAVE-button.<br>"
				"The tags are stored in one simple text file, tags.txt, within the root directory. Therefore you must have write access to this directory to add, edit or delete tags.<br>"
				"An existing file can still be read as long as you have read access, but appearently you can not make changes to the file then.<br>"
				"In this file the '|' character is used as the field separator, so this character cannot be used as part of a tag.<br>"
				"This will be checked when entering a tag. '|' will be replaced by '_' automatically, but you will be informed about his.<p>"
				"Select a filter tag from one of the comboboxes at the top of the screen to filter the list of movies.<p>"
				"<a href=""#Top"">back to top</a><p>"

				"<hr>"
				"<a name=""Options""><b><u>Command line options</u></b></a><p>"
				"Starting filmat11 with any command line argument will give you a help/version information message on standard out (the console).<p>"
				"<a href=""#Top"">back to top</a><p>"
                  
				"<hr>"
				"<a name=""Faq""><b><u>FAQ</u></b></a><p>"
				"<b>What about the name - filmat11 ?</b><br>"
				"It's a reminiscent to an old Monty Python sketch, from memory it goes somehow like : 'We interrupt this program to annoy you and make things generally irritating. Film at eleven.' British humo(u)r...<p>"
				"<a href=""#Top"">back to top</a><p>"

				"<hr>"
				"<a name=""Credits""><b><u>Credits</u></b></a><p>"
				"filmat11 uses mpv.<br>"
				"The program icon is made by me using inkscape. It is based on :<br>"
				" - cinemaparadisio.svg from openclipart.org, uploaded 2013.05.12, Cinema Paradisio, made by chatard, Public Domain<br>"
				" - font Awesome 6<br>"
				"Icons used within tlmb:<br>"
				"- about.svg - CapA.svg, openclipart.org, uploaded 2016-04-03 by AdamStanislav, Public Domain.<br>"
				"- help.svg - open_icon_library-GPL-0.10, actions/help-contents-3.svg updated by H. Kortendieck (me).<br>"
				"- open.svg - actions/document-open-5.svg, open icon library version 2009-11-10, by tango (team), Public Domain.<br>"
				"- quit.svg - open_icon_library-GPL-0.10, actions/system-shutdown-6.svg .<br>"
				"- settings.svg - papapishu-Tools-1.svg, openclipart.org, uploaded 2007-12-28 by papapishu, Public Domain.<br>"
				"- tag.svg - tag.svg, openclipart.org, uploaded 2010-09-06 by jhnri4, Public Domain.<br>"
				"Program and Documentation Copyright 2010-2023 Helge Kortendieck, alan@kasmithee.de .<p>"
				"filmat11 is released under the MIT license.<p>"
				"<a href=""#Top"">back to top</a><p>"
				"</body></html>");
			break;
	}
}
