/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2010-2024 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *                                                                   *
 *  Core logic of the program, calls other classes:                  *
 *  - playerproc      player process show a movie via mpv			 *
 *  - rw              read/write files                               *
 *  - settings        program settings dialog						 *
 *  - tags			  edit movie tags dialog						 *
 *********************************************************************/
#include "filmat11.h"

Fa11::Fa11(QMainWindow *parent)
        : QMainWindow(parent)
{
	uiFa11.setupUi(this);
	setWindowFlags(Qt::FramelessWindowHint);
	
//	QIcon::setThemeName("default");
	this->setWindowIcon(QIcon("/usr/share/icons/hicolor/scalable/apps/filmat11.svg"));

	QScreen *screen = QGuiApplication::primaryScreen();
	rFrame = screen->geometry();
	iScrX = rFrame.width();								// screen size horizontal (pixels)
	iScrY = rFrame.height();							// screen size vertical (pixels)

	showMaximized();

	playerCreate();
	emit sigFa11PlayerChkAvail();						// check for external player program(s) - currently only for mpv - continue in playerProgsAvail() with init() if one found
}

Fa11::~Fa11()
{
}


void Fa11::init()									// start things up
{
	iWShown = 0;
	strFilter.iType = 0;
	strFilter.sText = "";

	settingsRead();

	actionsCreate();

	uiFa11.fa11_tw_Movies->setContextMenuPolicy(Qt::CustomContextMenu);
	twMoviesHeaderSet();
	twMoviesMenuCreate();
	twMoviesCon(true);

	uiFa11.pb_Settings->setIcon(QIcon("/usr/share/filmat11/icons/scalable/settings.svg"));
	uiFa11.pb_Help->setIcon(QIcon("/usr/share/filmat11/icons/scalable/help.svg"));
	uiFa11.pb_About->setIcon(QIcon("/usr/share/filmat11/icons/scalable/about.svg"));
	uiFa11.pb_Quit->setIcon(QIcon("/usr/share/filmat11/icons/scalable/quit.svg"));

	con(true);

	timersCreate();
	
	iMovieCurIdx = 0;
	sMovieCurName = "";
	liStrMoviesGet();								// load/get list with movie structs
	filterCbsPop();
	filterApply();
}

void Fa11::settingsRead()							// read settings
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("mainWindow");
//    QPoint pos = settings.value( "pos", QPoint(0, 0) ).toPoint();
//    QSize size = settings.value( "size", QSize(800, 600) ).toSize();
		iColWidth0 = settings.value("twMoviesWidthCol0", 50).toInt(); 			// movies table widget: width column 0, name
		iColWidth1 = settings.value("twMoviesWidthCol1", 50).toInt(); 			// movies table widget: width column 1, genre
		iColWidth2 = settings.value("twMoviesWidthCol2", 50).toInt(); 			// movies table widget: width column 2, lang1
		iColWidth3 = settings.value("twMoviesWidthCol3", 50).toInt(); 			// movies table widget: width column 3, lang2
		iColWidth4 = settings.value("twMoviesWidthCol4", 50).toInt(); 			// movies table widget: width column 4, tag1
		iColWidth5 = settings.value("twMoviesWidthCol5", 50).toInt(); 			// movies table widget: width column 5, tag2
		iColWidth6 = settings.value("twMoviesWidthCol6", 50).toInt(); 			// movies table widget: width column 6, tag3
		iColWidth7 = settings.value("twMoviesWidthCol7", 50).toInt(); 			// movies table widget: width column 7, tag4
		iColWidth8 = settings.value("twMoviesWidthCol8", 50).toInt(); 			// movies table widget: width column 8, size GB
	settings.endGroup();

	settings.beginGroup("settings");
	    sRootDir = settings.value("rootDir", "").toString();	// root dir with movies
	settings.endGroup();

/*		iSize = settings.beginReadArray("subdirs");
		    qlSubDirs.clear();
			for (int i = 0; i < iSize; ++i) {
				settings.setArrayIndex(i);
				stSubDir.subdir = settings.value("subdir").toString();
				stSubDir.label = settings.value("label").toString();
				qlSubDirs.append(stSubDir);
			}
			settings.endArray();
*/

//  resize(size);
//  move(pos);
}

void Fa11::settingsWrite()							// write settings
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("mainWindow");
//		settings.setValue("tab", uiM.tw_Main->currentIndex());				// active tab
//    settings.setValue( "pos", pos() );
//    settings.setValue( "size", size() );
		settings.setValue("twMoviesWidthCol0", uiFa11.fa11_tw_Movies->columnWidth(0));		// movies table widget: width column 0, name
		settings.setValue("twMoviesWidthCol1", uiFa11.fa11_tw_Movies->columnWidth(1));		// movies table widget: width column 1, genre
		settings.setValue("twMoviesWidthCol2", uiFa11.fa11_tw_Movies->columnWidth(2));		// movies table widget: width column 2, lang1
		settings.setValue("twMoviesWidthCol3", uiFa11.fa11_tw_Movies->columnWidth(3));		// movies table widget: width column 3, lang2
		settings.setValue("twMoviesWidthCol4", uiFa11.fa11_tw_Movies->columnWidth(4));		// movies table widget: width column 4, tag1
		settings.setValue("twMoviesWidthCol5", uiFa11.fa11_tw_Movies->columnWidth(5));		// movies table widget: width column 5, tag2
		settings.setValue("twMoviesWidthCol6", uiFa11.fa11_tw_Movies->columnWidth(6));		// movies table widget: width column 6, tag3
		settings.setValue("twMoviesWidthCol7", uiFa11.fa11_tw_Movies->columnWidth(7));		// movies table widget: width column 7, tag4
		settings.setValue("twMoviesWidthCol8", uiFa11.fa11_tw_Movies->columnWidth(8));		// movies table widget: width column 8, size GB
	settings.endGroup();

//	settings.beginGroup("settings");
/*		settings.beginWriteArray("subdirs");
		    for (int i = 0; i < uiM.tw_MainWinSetSubDir->rowCount(); ++i) {
				settings.setArrayIndex(i);
				settings.setValue("subdir", uiM.tw_MainWinSetSubDir->item(i, 0)->text());		// ************* change to read pairs from widget!
				settings.setValue("label", uiM.tw_MainWinSetSubDir->item(i, 1)->text());
			}
			settings.endArray();
*/
//	settings.endGroup();

	settings.sync();
}


void Fa11::con(bool bD)								// connect (true) / disconnect (false) main widget signals
{
	if (bD == true) {
		connect(uiFa11.pb_About, SIGNAL(clicked()), this, SLOT(wAhClickedAbout()));		// show about dialog
		connect(uiFa11.pb_Help, SIGNAL(clicked()), this, SLOT(wAhClickedHelp()));		// show online help
		connect(uiFa11.pb_Quit, SIGNAL(clicked()), this, SLOT(quitPrep()));				// quit (prep)
		connect(uiFa11.pb_Settings, SIGNAL(clicked()), this, SLOT(wSetDlgCreate()));	// show settings dialog

//		connect(uiFa11.pb_FilterClear, SIGNAL(clicked()), this, SLOT(wSetDlgClicked()));	// clear filter settings
//		connect(uiFa11.pb_FilterApply, SIGNAL(clicked()), this, SLOT(wSetDlgClicked()));	// apply filter settings ************ needed? react on changes on comboboxes
		
//		connect(uiFa11.cb_FilterLang, SIGNAL(clicked()), this, SLOT(wSetDlgClicked()));	// show settings dialog
//		connect(uiFa11.cb_FilterGenre, SIGNAL(clicked()), this, SLOT(wSetDlgClicked()));	// show settings dialog
//		connect(uiFa11.cb_FilterTag, SIGNAL(clicked()), this, SLOT(wSetDlgClicked()));	// show settings dialog
	} else {
		
	}
}
/*
void Fa11::mainSetUpSettingsTab()			// set up settings tab
{
//	uiM.pb_MainWinSetMainDirReScan->setIcon(QIcon("/usr/share/filmat11/icons/scalable/refresh.svg"));	// main dir icons
//	uiM.pb_MainWinSetMainDirFileDlg->setIcon(QIcon("/usr/share/filmat11/icons/scalable/open.svg"));
//	connect(uiM.pb_MainWinSetMainDirReScan, SIGNAL(clicked()), this, SLOT(settingsMainDirReScan()));	// main dir re-scan
//	connect(uiM.pb_MainWinSetMainDirFileDlg, SIGNAL(clicked()), this, SLOT(settingsMainDirFileDlg()));	// main dir file dialog

//	uiM.tw_MainWinSetSubDir->horizontalHeader()->setStretchLastSection(true);							// sub dirs
	settingsSubDirsPopulate();							// populate the sub dirs QTableWidget with data loaded at start
	settingsMainDirReScan();							// re-scan if old dirs are still there and/or if new dirs in main dir
}
*/

void Fa11::quitPrep()								// quit clicked clean up
{
	settingsWrite();
	/*
	qPos = QCursor::pos();
	iD = qPos.x() - 175;
	if( iD < 0 ) {
		iD = 0;
	}
	iD2 = qPos.y() - 175;
	if( iD2 < 0 ) {
		iD2 = 0;
	}
	QMessageBox msgBox;
	msgBox.setWindowTitle( tr( "Quit ?" ) );
	msgBox.setIcon( QMessageBox::Question );
	msgBox.setText( tr( "\nSure to quit ?\n" ) );
	msgBox.addButton( tr( "Yes, &Quit" ), QMessageBox::AcceptRole );
	msgBox.addButton( tr( "No, &Cancel" ), QMessageBox::RejectRole );
	msgBox.move( iD, iD2 );
	if( msgBox.exec() == 0 ) {
		bQuit = true;
		ui.pB_Quit->setEnabled( false );        // disable quit button
		settingsWrite();
		unMnt();
	}
	*/
	quitDo();
}

void Fa11::quitDo()									// finally quit the program
{
	exit(0);
}


void Fa11::actionsCreate()							// create actions
{
	actTagsShowWdgt = new QAction(QIcon("/usr/share/filmat11/icons/scalable/tag.svg"), tr("Edit movie tags"), this);
	actTagsShowWdgt->setStatusTip(tr("Show movie tags editing dialog"));
	connect(actTagsShowWdgt, SIGNAL(triggered()), this, SLOT(wTagDlgCreate()));
}


// **********************************************
// **  movie filter related functions          **
// **********************************************
void Fa11::filterApply()							// apply current filter settings
{
	filterGenreCbCon(false);
	filterLangCbCon(false);
	filterTagCbCon(false);
	switch (strFilter.iType) {
		case 0:											// no filter, show all
			uiFa11.cb_FilterGenre->setCurrentText("");
			uiFa11.cb_FilterLang->setCurrentText("");
			uiFa11.cb_FilterTag->setCurrentText("");
			liStrMoviesFiltered = liStrMovies;
			break;
		case 1:
			uiFa11.cb_FilterLang->setCurrentText("");
			uiFa11.cb_FilterTag->setCurrentText("");
			liStrMoviesFiltered = filterLiMoviesGet();
			break;
		case 2:
			uiFa11.cb_FilterGenre->setCurrentText("");
			uiFa11.cb_FilterTag->setCurrentText("");
			liStrMoviesFiltered = filterLiMoviesGet();
			break;
		case 3:
			uiFa11.cb_FilterGenre->setCurrentText("");
			uiFa11.cb_FilterLang->setCurrentText("");
			liStrMoviesFiltered = filterLiMoviesGet();
			break;		
	}
	filterGenreCbCon(true);
	filterLangCbCon(true);
	filterTagCbCon(true);
	twMoviesPop();
}

void Fa11::filterCbsPop()							// populate filter comboboxes with tags - genre, language, tag
{
	strTagsListsD = rwTagsListsGet(liStrMovies);

	filterGenreCbCon(false);
	sD = uiFa11.cb_FilterGenre->currentText();
	uiFa11.cb_FilterGenre->clear();
	uiFa11.cb_FilterGenre->addItems(strTagsListsD.slGenres);
	uiFa11.cb_FilterGenre->setCurrentText(sD);
	filterGenreCbCon(true);

	filterLangCbCon(false);
	sD = uiFa11.cb_FilterLang->currentText();
	uiFa11.cb_FilterLang->clear();
	uiFa11.cb_FilterLang->addItems(strTagsListsD.slLangs);
	uiFa11.cb_FilterLang->setCurrentText(sD);
	filterLangCbCon(true);

	filterTagCbCon(false);
	sD = uiFa11.cb_FilterTag->currentText();
	uiFa11.cb_FilterTag->clear();
	uiFa11.cb_FilterTag->addItems(strTagsListsD.slTags);
	uiFa11.cb_FilterTag->setCurrentText(sD);
	filterTagCbCon(true);
}

void Fa11::filterGenreCbCon(bool bD)				// connect (true) / disconnect (false) genre filter combobox signals
{
	if (bD == true) {
		connect(uiFa11.cb_FilterGenre, SIGNAL(currentIndexChanged(int)), this, SLOT(filterGenreChgd(int)));
	} else {
		disconnect(uiFa11.cb_FilterGenre, SIGNAL(currentIndexChanged(int)), this, SLOT(filterGenreChgd(int)));
	}
}

void Fa11::filterGenreChgd(int iD)					// filter genre changed
{
	strFilter.sText = uiFa11.cb_FilterGenre->currentText();
	if (strFilter.sText == "") {
		strFilter.iType = 0;
	} else {
		strFilter.iType = 1;
	}
	filterApply();
}

void Fa11::filterLangCbCon(bool bD)					// connect (true) / disconnect (false) language filter combobox signals
{
	if (bD == true) {
		connect(uiFa11.cb_FilterLang, SIGNAL(currentIndexChanged(int)), this, SLOT(filterLangChgd(int)));
	} else {
		disconnect(uiFa11.cb_FilterLang, SIGNAL(currentIndexChanged(int)), this, SLOT(filterLangChgd(int)));
	}
}

void Fa11::filterLangChgd(int iD)					// filter language changed
{
	strFilter.sText = uiFa11.cb_FilterLang->currentText();
	if (strFilter.sText == "") {
		strFilter.iType = 0;
	} else {
		strFilter.iType = 2;
	}
	filterApply();
}

QList<StrMovie> Fa11::filterLiMoviesGet()			// get filtered movie list
{
	liStrMoviesFiltered.clear();
	if (liStrMovies.count() > 0) {
		switch (strFilter.iType) {
			case 1:										// movie list filtered by genre
				for (i = 0; i < liStrMovies.count(); ++i) {
					strMovieD = liStrMovies.at(i);
					if (strMovieD.sGenre == strFilter.sText) {
						liStrMoviesFiltered.append(strMovieD);
					}
				}
				break;
			case 2:										// movie list filtered by language
				for (i = 0; i < liStrMovies.count(); ++i) {
					strMovieD = liStrMovies.at(i);
					if (strMovieD.sLang1 == strFilter.sText or strMovieD.sLang2 == strFilter.sText) {
						liStrMoviesFiltered.append(strMovieD);
					}
				}
				break;
			case 3:										// movie list filtered by tag
				for (i = 0; i < liStrMovies.count(); ++i) {
					strMovieD = liStrMovies.at(i);
					if (strMovieD.sTag1 == strFilter.sText or strMovieD.sTag2 == strFilter.sText or strMovieD.sTag3 == strFilter.sText or strMovieD.sTag4 == strFilter.sText) {
						liStrMoviesFiltered.append(strMovieD);
					}
				}
				break;
		}
	}
	return liStrMoviesFiltered;
}

void Fa11::filterTagCbCon(bool bD)					// connect (true) / disconnect (false) tag filter combobox signals
{
	if (bD == true) {
		connect(uiFa11.cb_FilterTag, SIGNAL(currentIndexChanged(int)), this, SLOT(filterTagChgd(int)));
	} else {
		disconnect(uiFa11.cb_FilterTag, SIGNAL(currentIndexChanged(int)), this, SLOT(filterTagChgd(int)));
	}
}

void Fa11::filterTagChgd(int iD)					// filter tag changed
{
	strFilter.sText = uiFa11.cb_FilterTag->currentText();
	if (strFilter.sText == "") {
		strFilter.iType = 0;
	} else {
		strFilter.iType = 3;
	}
	filterApply();
}


// **********************************************
// **  functions on struct lists               **
// **********************************************
void Fa11::liStrMoviesGet()							// load/get list with movie structs
{
	if (rwDirStatus(sRootDir) > 109) {					// readable root dir
		liStrMovies = rwMoviesListStrGet(sRootDir);
	} else {											// root dir invalid
		mbTxt_RootDirInvalid();
		return;
	}
}

int Fa11::liStrMoviesIdxGet(const QString &sD)		// get index in master list of current movie
{
	iD = -1;
	if (liStrMovies.count() > 0) {
		for (i = 0; i < liStrMovies.count(); ++i) {	// iterate over song list and find name match
			strMovieD = liStrMovies.at(i);
			if (sD == strMovieD.sName) {
				iD = i;
				break;
			}
		}
	}
	return iD;
}

int Fa11::liStrMoviesFilteredIdxGet(const QString &sD)	// get index in master list of current movie
{
	iD = -1;
	if (liStrMoviesFiltered.count() > 0) {
		for (i = 0; i < liStrMoviesFiltered.count(); ++i) {		// iterate over song list and find name match
			strMovieD = liStrMoviesFiltered.at(i);
			if (sD == strMovieD.sName) {
				iD = i;
				break;
			}
		}
	}
	return iD;
}


// **********************************************
// **  message box and it's messages           **
// **********************************************
int Fa11::mbShow(int iD, const QString& sD, const QString& sD2, const QString& sD3, const QString& sD4, const QString& sD5)
{						// show QMessageBox at cursor position: type, title, text, button accept, button reject, button destruct
	QMessageBox msgBox;
	msgBox.setWindowTitle(sD);

	if (iD == 1) {
		msgBox.setIcon(QMessageBox::Information);
	} else if (iD == 2) {
		msgBox.setIcon(QMessageBox::Warning);
	} else if (iD == 3) {
		msgBox.setIcon(QMessageBox::Critical);
	} else if (iD == 4) {
		msgBox.setIcon(QMessageBox::Question);	// 2 buttons
	} else if (iD == 5) {
		msgBox.setIcon(QMessageBox::Question);	// 3 buttons, 0=yes, 1=cancel, 2=no
	} else {
		return 9;
	}

	msgBox.setText(sD2);

	msgBox.addButton(sD3, QMessageBox::AcceptRole);
	if (iD == 5) {
		msgBox.addButton(sD5, QMessageBox::DestructiveRole);
	}
	if (iD > 3) {
		msgBox.addButton(sD4, QMessageBox::RejectRole);
	}

	iX = iScrX/2 - msgBox.frameGeometry().width()/2;		// this adjustment should center the mbShow - but it doesn't !?
	if (iX < 0) {
		iX = 0;
	}
	iY = iScrY/2 - msgBox.frameGeometry().height()/2;
	if (iY < 0) {
		iY = 0;
	}
	msgBox.move(iX, iY);

	iRet = msgBox.exec();
	return iRet;
}

void Fa11::mbTxt_PlayerNoneFound()					// show message box - no supported player found
{
	mbShow(2, tr("Warning - no supported player found !"),
	          tr("Attention, none of the supported media players could be found.\n"
				 "Well, currently only mpv is supported.\n"
				 "So   /usr/bin/mpv   could not be found.\n \n"
				 "Without an external media player, no video can be played.\n"
				 "Aborting..."), tr("&OK"), "", "");
}

void Fa11::mbTxt_RootDirInvalid()					// show message box - root dir invalid
{
	mbShow(1, tr("Information - root directory invalid"),
	          tr("The root directory\n"
				 "%1\n"
				 "with the movies is invalid.\n"
				 "It does either not exist, is not a directory or is unreadable.\n"
				 "Please set the root directory in the settings dialog.").arg(sRootDir), tr("&OK"), "", "");
}

void Fa11::mbTxt_RootDirNoMovies()					// show message box - root dir no supported movies
{
	mbShow(1, tr("Information - no movies in root directory"),
	          tr("There are no supported movies in the root directory\n"
				 "%1\n"
				 "You may want to set a different root directory in the settings dialog.").arg(sRootDir), tr("&OK"), "", "");
}

void Fa11::mbTxt_TagsTxtSaved()						// show message box - tags.txt saved
{
	mbShow(1, tr("filmat11 information - tags.txt saved !"),slD.at(1) , tr("&OK"), "", "");
}

void Fa11::mbTxt_TagsTxtSavedNot()					// show message box - tags.txt NOT saved
{
	mbShow(1, tr("filmat11 warning - tags.txt NOT saved !"),slD.at(1) , tr("&OK"), "", "");
}


// ***********************************************************
// **  key press event handling                             **
// ***********************************************************

void Fa11::keyPressEvent(QKeyEvent *ke)	// key press event
{
//cout << "Tlmw::keyPressEvent(): reached !" << endl;                                               // *****  debug  *****
	switch (ke->keyCombination()) {
		case QKeyCombination(Qt::ShiftModifier, Qt::Key_F1):	// open filter by genre combobox
			uiFa11.cb_FilterGenre->showPopup();
			break;
		case QKeyCombination(Qt::ShiftModifier, Qt::Key_F2):	// open filter by language combobox
			uiFa11.cb_FilterLang->showPopup();
			break;
		case QKeyCombination(Qt::ShiftModifier, Qt::Key_F3):	// open filter by tag combobox
			uiFa11.cb_FilterTag->showPopup();
			break;
		case Qt::Key_MediaPlay:									// play selected song
			playerStart();
			break;
	}
}


// ************************************************************************
// **  player functions - external player, currently only mpv supported  **
// ************************************************************************
void Fa11::playerCreate()           				// create player qprocess, once at program start
{
	playerProc = new PlayerProc();
	connect(playerProc, SIGNAL(sigPpChkEnd(int)), this, SLOT(playerProgsAvail(int)));							// return from checking for available players, 1=mpv only one currently supported
	connect(playerProc, SIGNAL(sigPpExited(int)), this, SLOT(playerEnded(int)));								// qprocess playing a video ended: 0=normally, 1=crash
	connect(this, SIGNAL(sigFa11PlayerChkAvail()), playerProc, SLOT(ppChkAvail()));								// check for external player program(s) - currently only for mpv
	connect(this, SIGNAL(sigFa11PlayerStart(const QString&)), playerProc, SLOT(ppProcPrep(const QString&)));	// start playing video with external player
}

void Fa11::playerStart()           					// start player qprocess
{
	strMovieD = liStrMovies.at(iMovieCurIdx);
	emit sigFa11PlayerStart(strMovieD.sPath + "/" + strMovieD.sName + "." + strMovieD.sSuffix);
}

void Fa11::playerEnded(int iD) 						// player qprocess ended - 0=normal, 1=crash
{
	if (iD == 0) {									// normal exit

	} else {										// crash
		
	}
}

void Fa11::playerProgsAvail(int iD)					// receive result of checking available external players, 1=mpv (10=vlc, 11=both - currently only mpv is supported)
{
	iPlayers = iD;
//cout << "playerProgsAvail reached " << QString::number(iPlayers).toStdString() << endl;                                               // *****  debug
	if (iPlayers > 0) {									// supported player found - currently only mpv supported
		init();
	} else {											// no supported player found
		mbTxt_PlayerNoneFound();
		quitPrep();
	}
}


// ********************************************************
// **  timers											 **
// ********************************************************
void Fa11::timersCreate()							// create all timers
{
	timerSaveHeaderWidth = new QTimer(this);										// 5 sec timer to limit checks if the table column headers where resized
	timerSaveHeaderWidth->setSingleShot(true);
	connect(timerSaveHeaderWidth, SIGNAL(timeout()), this, SLOT(settingsWrite()));	// save new column width of history table widget
}

void Fa11::timerSaveHeaderWidthStart()				// start timer to save header width of all qtablewidgets
{
	if (timerSaveHeaderWidth->isActive() == false) {
		timerSaveHeaderWidth->start(5000);
	}
}


// ****************************************
// **  movies table widget functions     **
// ****************************************
void Fa11::twMoviesCellActivated(int iD, int iD2)	// cell in table widget activated (clicked) (row, col)
{
	twItemD = uiFa11.fa11_tw_Movies->item(iD, 0);		// row, col
	sMovieCurName = twItemD->text();
	iMovieCurIdx = liStrMoviesIdxGet(sMovieCurName);
//cout << "twMoviesCellActivated - iMovieCurIdx = " << QString::number(iMovieCurIdx).toStdString() << endl;                               // *****  debug  *****
}

void Fa11::twMoviesCellCurChgd(int iD, int iD2, int iD3, int iD4)	// current cell changed - (cur row,col) (old row,col) y,x y,x
{
	twItemD = uiFa11.fa11_tw_Movies->item(iD, 0);						// row, col
	sMovieCurName = twItemD->text();
	iMovieCurIdx = liStrMoviesIdxGet(sMovieCurName);
//cout << "twMoviesCellCurChgd - iMovieCurIdx = " << sMovieCurName.toStdString() << " ; " << QString::number(iMovieCurIdx).toStdString() << endl;                               // *****  debug  *****
//	iMovieCurIdx = iD;
}

void Fa11::twMoviesCellDoubleClicked(int iD, int iD2)	// cell in table widget double clicked (row, col) - play video
{
	twItemD = uiFa11.fa11_tw_Movies->item(iD, 0);			// row, col
	sMovieCurName = twItemD->text();
	iMovieCurIdx = liStrMoviesIdxGet(sMovieCurName);
//cout << "twMoviesCellDoubleClicked - iMovieCurIdx = " << QString::number(iMovieCurIdx).toStdString() << endl;                               // *****  debug  *****
//	iMovieCurIdx = iD;
	playerStart();
}

void Fa11::twMoviesClear()							// clear table widget videos
{
	uiFa11.fa11_tw_Movies->clearContents();
	uiFa11.fa11_tw_Movies->setRowCount(0);
//	sPlaySong = "";
	iMoviesCnt = 0;
//	iCurRowIx = -1;
//	iCurCol = -1;
}

void Fa11::twMoviesCon(bool bD)						// connect (true) / disconnect (false) videos table widget signals
{
	if (bD == true) {
		connect(uiFa11.fa11_tw_Movies->horizontalHeader(), SIGNAL(sectionResized(int, int, int)), this, SLOT(timerSaveHeaderWidthStart()));		// 5sec delay: check col headers w resized->save
		connect(uiFa11.fa11_tw_Movies, SIGNAL(cellActivated(int, int)), this, SLOT(twMoviesCellDoubleClicked(int, int)));						// song in table widget activated (ENTER pressed) - play it
		connect(uiFa11.fa11_tw_Movies, SIGNAL(cellDoubleClicked(int, int)), this, SLOT(twMoviesCellDoubleClicked(int, int)));					// song in table widget double clicked - play it
		connect(uiFa11.fa11_tw_Movies, SIGNAL(currentCellChanged(int, int, int, int)), this, SLOT(twMoviesCellCurChgd(int, int, int, int)));	// current cell changed, set (new?) row index
		connect(uiFa11.fa11_tw_Movies, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(twMoviesMenuCalled(const QPoint&)));
	} else {
		disconnect(uiFa11.fa11_tw_Movies->horizontalHeader(), SIGNAL(sectionResized(int, int, int)), this, SLOT(timerSaveHeaderWidthStart()));	// 5sec delay: check col headers w resized->save
		disconnect(uiFa11.fa11_tw_Movies, SIGNAL(cellActivated(int, int)), this, SLOT(twMoviesCellDoubleClicked(int, int)));					// song in table widget activated (ENTER pressed) - play it
		disconnect(uiFa11.fa11_tw_Movies, SIGNAL(cellDoubleClicked(int, int)), this, SLOT(twMoviesCellDoubleClicked(int, int)));				// song in table widget double clicked - play it
		disconnect(uiFa11.fa11_tw_Movies, SIGNAL(currentCellChanged(int, int, int, int)), this, SLOT(twMoviesCellCurChgd(int, int, int, int)));	// current cell changed, set (new?) row index
		disconnect(uiFa11.fa11_tw_Movies, SIGNAL(customContextMenuRequested(const QPoint&)), this, SLOT(twMoviesMenuCalled(const QPoint&)));
	}
}

void Fa11::twMoviesHeaderSet()						// set movie's table widget header
{
	uiFa11.fa11_tw_Movies->setColumnCount(9);
	uiFa11.fa11_tw_Movies->horizontalHeader()->setDefaultAlignment(Qt::AlignHCenter);
	uiFa11.fa11_tw_Movies->horizontalHeader()->setStretchLastSection(true);
	slD << tr("Movie name") << tr("Genre") << tr("Lang 1") << tr("Lang 2") << tr("Tag 1") << tr("Tag 2") << tr("Tag 3") << tr("Tag 4") << tr("Size GB");
	uiFa11.fa11_tw_Movies->setHorizontalHeaderLabels(slD);

	uiFa11.fa11_tw_Movies->setColumnWidth(0, iColWidth0);
	uiFa11.fa11_tw_Movies->setColumnWidth(1, iColWidth1);
	uiFa11.fa11_tw_Movies->setColumnWidth(2, iColWidth2);
	uiFa11.fa11_tw_Movies->setColumnWidth(3, iColWidth3);
	uiFa11.fa11_tw_Movies->setColumnWidth(4, iColWidth4);
	uiFa11.fa11_tw_Movies->setColumnWidth(5, iColWidth5);
	uiFa11.fa11_tw_Movies->setColumnWidth(6, iColWidth6);
	uiFa11.fa11_tw_Movies->setColumnWidth(7, iColWidth7);
	uiFa11.fa11_tw_Movies->setColumnWidth(8, iColWidth8);
	uiFa11.fa11_tw_Movies->setAlternatingRowColors(true);
}

void Fa11::twMoviesMenuCalled(const QPoint& pD)		// right click in movies table widget - call context menu
{
//	menuTwMoviesRightClick->exec(QCursor::pos());
	wTagDlgCreate();
}

void Fa11::twMoviesMenuCreate()						// create movie's table widget right-click menu
{
	menuTwMoviesRightClick = new QMenu;
	menuTwMoviesRightClick->clear();
	menuTwMoviesRightClick->addAction(actTagsShowWdgt);
}

void Fa11::twMoviesPop()							// populate movies widget
{
//cout << "fa11: twMoviesPop - reached " << endl;                               // *****  debug  *****
	twMoviesCon(false);
	uiFa11.fa11_tw_Movies->clearContents();

	iMoviesCnt = liStrMoviesFiltered.count();
//cout << "fa11: twMoviesPop - liStrMoviesFiltered.count() = " << QString::number(iMoviesCnt).toStdString() << endl;                               // *****  debug  *****
	if (iMoviesCnt > 0) {
		uiFa11.fa11_tw_Movies->setRowCount(iMoviesCnt);
		for (i = 0; i < iMoviesCnt; ++i) {				// iterate over song list and populate widget
			strMovieD = liStrMoviesFiltered.at(i);
			for (i2 = 0; i2 < 9; ++i2) {
				switch (i2) {
					case 0:
						twItemD = new QTableWidgetItem(strMovieD.sName);
						twItemD->setTextAlignment(Qt::AlignLeft);
//cout << "fa11: twMoviesPop - name, i " << strMovieD.sName.toStdString() << QString::number(i).toStdString() << endl;                               // *****  debug  *****
						break;
					case 1:
						twItemD = new QTableWidgetItem(strMovieD.sGenre);
						twItemD->setTextAlignment(Qt::AlignLeft);
						break;
					case 2:
						twItemD = new QTableWidgetItem(strMovieD.sLang1);
						twItemD->setTextAlignment(Qt::AlignCenter);
						break;
					case 3:
						twItemD = new QTableWidgetItem(strMovieD.sLang2);
						twItemD->setTextAlignment(Qt::AlignCenter);
						break;
					case 4:
						twItemD = new QTableWidgetItem(strMovieD.sTag1);
						twItemD->setTextAlignment(Qt::AlignLeft);
						break;
					case 5:
						twItemD = new QTableWidgetItem(strMovieD.sTag2);
						twItemD->setTextAlignment(Qt::AlignLeft);
						break;
					case 6:
						twItemD = new QTableWidgetItem(strMovieD.sTag3);
						twItemD->setTextAlignment(Qt::AlignLeft);
						break;
					case 7:
						twItemD = new QTableWidgetItem(strMovieD.sTag4);
						twItemD->setTextAlignment(Qt::AlignLeft);
						break;
					case 8:
						twItemD = new QTableWidgetItem(QString::number(strMovieD.fSize, 'f', 2));
						twItemD->setTextAlignment(Qt::AlignCenter);
						break;
				}
				uiFa11.fa11_tw_Movies->setItem(i, i2, twItemD);
			}
		}
		uiFa11.fa11_tw_Movies->setCurrentCell(liStrMoviesFilteredIdxGet(sMovieCurName), 0);
//??		twMoviesCellActivated(0,0);
		uiFa11.fa11_tw_Movies->setFocus();
	} else {										// no supported movie files in root dir
			mbTxt_RootDirNoMovies();
	}

	twMoviesCon(true);
}


// ***********************************************************
// **  about help dialog widget handling                    **
// ***********************************************************
void Fa11::wAhClickedAbout()						// clicked show about, 2
{
	wAhCreate(2);
}

void Fa11::wAhClickedHelp()							// clicked show online help, 3
{
	wAhCreate(3);	
}

void Fa11::wAhCreate(int iD)						// create n show about/help dialog, 2=about 3=help
{
	if (iWShown == 0) {
		if (iD > 1 and iD < 4) {
			wAh = new AboutHelp();						// about help widget
			connect(this, SIGNAL(sigFa11WahShow(int)), wAh, SLOT(dlgShow(int)));	// show about/help widget
			connect(wAh, SIGNAL(sigWahClose()), this, SLOT(wAhDel()));				// about/help widget closed - destroy
			switch (iD) {
				case 2:
					iWShown = 2;
					emit sigFa11WahShow(2);
					break;
				case 3:
					iWShown = 3;
					emit sigFa11WahShow(3);
					break;
			}
		}
	}
}

void Fa11::wAhDel()									// close about/help dialog
{
	wAh->deleteLater();
	iWShown = 0;
}


// *********************************************************
// **  settings dialog widget handling                    **
// *********************************************************
void Fa11::wSetDlgCreate()							// create n show settings dialog
{
	if (iWShown == 0) {
		wSetDlg = new SetDlg();					// settings dialog widget
		connect(this, SIGNAL(sigFa11WsetDlgShow()), wSetDlg, SLOT(setDlgShow()));		// show settings dialog widget
		connect(wSetDlg, SIGNAL(sigSetDlgClose(bool)), this, SLOT(wSetDlgDel(bool)));	// settings dialog widget closed - destroy, true=re-read settings
		iWShown = 1;
		emit sigFa11WsetDlgShow();
	}
}

void Fa11::wSetDlgDel(bool bD)						// hide=delete settings dialog, true=re-read settings
{
	wSetDlg->deleteLater();
	iWShown = 0;
	if (bD == true) {
		settingsRead();
		liStrMoviesGet();								// load/get list with movie structs
		filterCbsPop();
		filterApply();
	}
}

// *********************************************************
// **  edit tags dialog widget handling                   **
// *********************************************************
void Fa11::wTagDlgCreate()							// create n show edit tag dialog
{
	if (iWShown == 0) {
		wTagDlg = new TagDlg();							// edit tag dialog widget
		connect(this, SIGNAL(sigFa11WtagDlgShow(QList<StrMovie>, int)), wTagDlg, SLOT(td_Show(QList<StrMovie>, int)));	// show edit tag dialog widget
		connect(wTagDlg, SIGNAL(sigTagDlgClose(bool, QList<StrMovie>)), this, SLOT(wTagDlgDel(bool, QList<StrMovie>)));		// edit tag dialog widget closed - destroy, true=re-read tags
		iWShown = 4;
		emit sigFa11WtagDlgShow(liStrMovies, iMovieCurIdx);
		wTagDlg->setFocus();							// do I need a timer here?
	}
}

void Fa11::wTagDlgDel(bool bD, QList<StrMovie> liStrMoviesD)	// hide=delete edit tags dialog - true=re-read tag (we have changes), movie str list, new current entry index
{
	wTagDlg->deleteLater();
	iWShown = 0;
	if (bD == true) {
		liStrMovies = liStrMoviesD;
//cout << "fa11: wTagDlgDel - reached " << endl;                               // *****  debug  *****
		slD = rwTagsFileSave(sRootDir, liStrMovies);				// update tags.txt
		if (slD.at(0) == "0") {
			mbTxt_TagsTxtSaved();
		} else {
			mbTxt_TagsTxtSavedNot();
		}
		filterCbsPop();
		filterApply();												// show updated tags in movies widget
	}
}
