/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2010-2024 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/
#ifndef _FILMAT11_H_
#define _FILMAT11_H_

#include <QAction>
#include <QDir>
#include <QFileDialog>
#include <QFileInfo>
#include <QHeaderView>
#include <QIcon>
#include <QKeyEvent>
#include <QList>
#include <QMainWindow>
#include <QMenu>
#include <QMessageBox>
#include <QPoint>
#include <QRect>
#include <QScreen>
#include <QSettings>
#include <QString>
#include <QTableWidget>
#include <QTableWidgetItem>
#include <QTimer>

/*
#include <iostream>
using namespace std;
*/

#include "abouthelp.h"
#include "playerproc.h"
#include "rw.h"
#include "settings.h"
#include "structs.h"
#include "tags.h"
//#include "video.h"

#include "ui_filmat11.h"

class Fa11 : public QMainWindow
{
  Q_OBJECT

public:
	Fa11(QMainWindow *parent = 0);
		~Fa11();

protected slots:
	void keyPressEvent(QKeyEvent *);

private:

	bool bD;
//	bool bQuit;           // flag whether we want to quit

	float fD;

	int i;
	int i2;
	int iColWidth0;
	int iColWidth1;
	int iColWidth2;
	int iColWidth3;
	int iColWidth4;
	int iColWidth5;
	int iColWidth6;
	int iColWidth7;
	int iColWidth8;
	int iD;					// dummy
//  int iD2;
	int iPlayers;			// available players, 1 = mpv (10=vlc, 11=both) - currently only mpv is supported)
	int iRet;				// return value of function (e. g. QMessageBox)
	int iScrX;				// screen width
	int iScrY;				// screen heigth
	int iSize;
	int iMovieCurIdx;		// index of current movie in list of movies
	int iMoviesCnt;			// number of movies in root dir
	int iWShown;			// widget currently shown: 0=none, 1=settings, 2=about, 3=help, 4=edit tag
	int iX;					// cursor position for messagebox
	int iY;					// cursor position for messagebox

	Ui::fa11_ui uiFa11;			// GUI

	QAction *actTagsShowWdgt;		// show widget to edit movie tags

	QMenu *menuTwMoviesRightClick;	// right-click menu for movies table widget

	QPoint pD;

	QRect rFrame;					// frame geometry = screen size

	QString sD;         	// dummy
	QString sMovieCurName;	// name of current movie
	QString sRootDir;		// root dir, full path & name, holding the movies
/*	QString sD2;
	QString sDir;         // directory with rsync definition
	QString sErr;         // error text
	QString sErrT;        // error dialog title
*/
	QStringList slD;				// dummy list

	StrFilter strFilter;			// struct of current filter setting: int type, QString text

	StrMovie strMovieD;						// movie struct entity
	QList<StrMovie> liStrMoviesD;			// list with movie structs for movies table widget
	QList<StrMovie> liStrMovies;			// list with movie structs for movies table widget
	QList<StrMovie> liStrMoviesFiltered;	// list with movie structs for movies table widget

	StrTagsLists strTagsListsD;		// tags lists struct entity - 3 tag lists: genre, language and tags

	QTableWidgetItem *twItemD;		// a QTableWidgetItem

	QTimer *timerSaveHeaderWidth;	// 5 sec timer to store resizing of column width of the qtablewidgets

	PlayerProc *playerProc;			// player qprocess
	
	AboutHelp *wAh;					// about/help widget
	SetDlg *wSetDlg;				// settings dialog
	TagDlg *wTagDlg;				// edit tag dialog

private slots:
	void actionsCreate();				// create actions

	void con(bool bD);					// connect (true) / disconnect (false) main widget signals

	void filterApply();						// apply current filter settings
	void filterCbsPop();					// populate filter comboboxes with tags - genre, language, tag
	void filterGenreCbCon(bool bD);			// connect (true) / disconnect (false) genre filter combobox signals
	void filterGenreChgd(int iD);			// filter genre changed
	void filterLangCbCon(bool bD);			// connect (true) / disconnect (false) language filter combobox signals
	void filterLangChgd(int iD);			// filter language changed
	QList<StrMovie> filterLiMoviesGet();	// get filtered movie list
	void filterTagCbCon(bool bD);			// connect (true) / disconnect (false) tag filter combobox signals
	void filterTagChgd(int iD);				// filter tag changed

	void init();						// start up

	void liStrMoviesGet();								// load/get list with movie structs
	int liStrMoviesIdxGet(const QString &sD);			// get current movie index in master list
	int liStrMoviesFilteredIdxGet(const QString &sD);	// get current movie index in filtered list

	int mbShow(int iD, const QString& sD, const QString& sD2, const QString& sD3, const QString& sD4, const QString& sD5);
										// show QMessageBox at cursor position, 1=Info, 2=Warning, 3=Critical, 4=Question, 5=Question yes/no/cancel
										// iD=type, sD=title, sD2=text, sD3=accept button text, sD4=reject button text, sD5=destructive button text (no but continue)
	void mbTxt_PlayerNoneFound();		// show message box - no supported player found
	void mbTxt_RootDirInvalid();		// show message box - root dir invalid
	void mbTxt_RootDirNoMovies();		// show message box - root dir no supported movies
	void mbTxt_TagsTxtSaved();			// show message box - tags.txt saved
	void mbTxt_TagsTxtSavedNot();		// show message box - tags.txt NOT saved

//	void dirChkRet( const QString& sD, const QString& sD2, int iD );  // return from directory check

	void playerCreate();				// create player qprocess, once at program start
	void playerEnded(int iD);			// player qprocess ended - 0=normal, 1=crashed
	void playerProgsAvail(int iD);		// receive result of checking available external players, 1=mpv (10=vlc, 11=both - currently only mpv is supported)
	void playerStart();                 // start player qprocess

	void quitPrep();					// quit clicked clean up, quit button was clicked
	void quitDo();						// finally quit the program

	void settingsRead();				// read settings (client mode data)
	void settingsWrite();				// write settings (client mode data)

	void timersCreate();				// create all timers
	void timerSaveHeaderWidthStart();	// start timer to save header width of all qtablewidgets

	void twMoviesCellActivated(int iD, int iD2);					// cell in table widget activated (clicked) (row, col)
	void twMoviesCellCurChgd(int iD, int iD2, int iD3, int iD4);	// current cell changed - set (new?) row index
	void twMoviesCellDoubleClicked(int iD, int iD2);				// cell in table widget double clicked (row, col) - play video
	void twMoviesClear();											// clear table widget videos
	void twMoviesCon(bool bD);										// connect (true) / disconnect (false) videos table widget signals
	void twMoviesHeaderSet();										// set movie's table widget header
	void twMoviesMenuCalled(const QPoint& pD);						// right click in table widget movies - call context menu
	void twMoviesMenuCreate();										// create movies table widget right-click menu
	void twMoviesPop();												// populate videos widget

	void wAhClickedAbout();				// clicked show about
	void wAhClickedHelp();				// clicked show online help
	void wAhCreate(int iD);				// create n show about/help dialog, 2=about, 3=help
	void wAhDel();						// hide=delete about/help dialog
	
	void wSetDlgCreate();				// create n show settings dialog
	void wSetDlgDel(bool bD);			// hide=delete settings dialog, true=re-read settings

	void wTagDlgCreate();				// create n show edit tag dialog
	void wTagDlgDel(bool bD, QList<StrMovie> liStrMoviesD);	// hide=delete edit tag dialog, true=re-read tags (we have changes)

	signals:
		void sigFa11WahShow(int iD);				// show about (1) or online  help (2)
		void sigFa11PlayerChkAvail();				// check for external player program(s) - currently only for mpv
		void sigFa11PlayerStart(const QString& sD);	// play this movie
		void sigFa11WsetDlgShow();					// show settings dialog
		void sigFa11WtagDlgShow(QList<StrMovie> liStrMovies, int iD);	// show edit tag dialog, iD = calling index
};

#endif

