/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2023 by Helge Kortendieck  alan@kasmithee.de		 *
 *                                                                   *
 *  run detached process with external media player                  *
 *  currently only mpv is supported (vlc may be later)               *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/

#include "playerproc.h"


PlayerProc::PlayerProc( QObject *parent ) : QObject( parent)
{
	pp_bPlaying = false;
}

PlayerProc::~PlayerProc()
{
}

void PlayerProc::ppChkAvail()					// check for player(s): found: 1=mpv (10=vlc not yet implemented, 11 = both)
{
	pp_iPlayers = 0;
	QFileInfo qfi( "/usr/bin/mpv" );				// check for mpv
	if (qfi.exists() == true) {
		pp_iPlayers = 1;
	}
//cout << "ppChkAvail reached" << QString::number(iPlayers).toStdString() << endl;                                               // *****  debug
	emit sigPpChkEnd(pp_iPlayers);					// send result of found players - currently only mpv supported
}

void PlayerProc::ppProcPrep(const QString& pp_sD)	// prepare player - full path n name of file to play
{
	pp_slProcArgs.clear();
	pp_slProcArgs.append(pp_sD);
	ppProcStart();
}

void PlayerProc::ppProcStart()					// run player process
{
	if (pp_bPlaying == false ) {
		ppPlayerProc = new QProcess(this);
		connect(ppPlayerProc, SIGNAL(readyReadStandardOutput()), this, SLOT(ppProcReadStdOut()));
		connect(ppPlayerProc, SIGNAL(readyReadStandardError()), this, SLOT(ppProcReadStdErr()));
		connect(ppPlayerProc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(ppProcExit(int, QProcess::ExitStatus)));
		pp_bPlaying = true;
		ppPlayerProc->start("mpv", pp_slProcArgs);
	} else {
		cout << "ppProcStart called, while pp_bPlaying = true " << endl;                                               // *****  debug
	}
}

void PlayerProc::ppProcReadStdOut()				// read Stdout of player process
{
	aOutput = ppPlayerProc->readAllStandardOutput();									// read byte array
	if (!aOutput.isEmpty()) {
//		cout << "filmat11: stdout: " << QString(aOutput).trimmed().toStdString() << endl;	// catch output line, mpv song info: video, AV, VO and exit status - currently disabled until there is use for the info
    }
}

void PlayerProc::ppProcReadStdErr()				// read Stderr of player process
{
	aOutput = ppPlayerProc->readAllStandardError();										// read byte array
	if (!aOutput.isEmpty()) {
//		cout << "filmat11: stderr: " << QString(aOutput).trimmed().toStdString() << endl;	// catch output line, currently disabled, mpv is 'spawning' stderr?
	}
}

void PlayerProc::ppProcExit(int iD, QProcess::ExitStatus iD2)	// player process has exited, 0=normal, 1=crash
{
	disconnect(ppPlayerProc, SIGNAL(readyReadStandardOutput()), this, SLOT(ppProcReadStdOut()));
	disconnect(ppPlayerProc, SIGNAL(readyReadStandardError()), this, SLOT(ppProcReadStdErr()));
	disconnect(ppPlayerProc, SIGNAL(finished( int, QProcess::ExitStatus)), this, SLOT(ppProcExit(int, QProcess::ExitStatus)));
	pp_bPlaying = false;
	if (ppPlayerProc->exitStatus() == QProcess::NormalExit) {   // normal exit
		emit sigPpExited(0);
    } else {                                    				// we crashed?
		cout << "filmat11: the player (mpv) process exited unexpected (crashed?) while playing: " << pp_sD.toStdString() << endl;                                           // *****  debug
		emit sigPpExited(1);
    }
}
