/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2023 by Helge Kortendieck  alan@kasmithee.de		 *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/

#ifndef _PLAYERPROC_H_
#define _PLAYERPROC_H_

#include <QByteArray>
#include <QDateTime>
#include <QFileInfo>
#include <QObject>
#include <QProcess>
#include <QString>

#include <iostream>
using namespace std;

class PlayerProc : public QObject
{
  Q_OBJECT

public:
	PlayerProc(QObject *parent = 0);
		~PlayerProc();

private:
	bool pp_bPlaying;				// track player qprocess running

//	int pp_iD;
	int pp_iPlayers;				// enum which players were found: 1 = mpv (not yet implemented: 10 = vlc, 11 = both)

	QByteArray aOutput;				// stdout qprocess

	QProcess *ppPlayerProc;			// detached player qprocess

	QString pp_sD;
//	QString pp_sIn;					// input line from player qprocess output

	QStringList pp_slProcArgs;		// qprocess argument(s)

private slots:
	void ppChkAvail();					// check for player(s): found: 1=mpv (10=vlc not yet implemented, 11 = both)

	void ppProcPrep(const QString& sD);	// prepare player - full path n name of file to play
	void ppProcStart();					// start rsync qprocess
	void ppProcExit(int iD, QProcess::ExitStatus iD2);	// player process has exited, 0=normal, 1=crash
	void ppProcReadStdErr();			// read error output from player process
	void ppProcReadStdOut();			// read output from player process

	signals:
		void sigPpChkEnd(int iPpD);		// checking for player(s) has ended. Result: 0=none installed, 1=mpv, 10=vlc not yet implemented (11 = both)
		void sigPpExited(int iPpD);		// player process exited: 0 = normally, 1 = unexpected/crash
};

#endif // _PLAYERPROC_H_
