/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2021-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/

#include "rw.h"

	float rw_fD;

	int rw_iBak;
	int rw_iBak2;
	int rw_iD;
	int rw_iD2;
	int rw_iD3;
	int rw_iFs;						// file status: 0= doesn't exists, +100=dir, +10=readable, +1=writable
	int rw_iFs2;					// file status: 0= doesn't exists, +100=dir, +10=readable, +1=writable
//	int iRwIx;						// index

	QString rw_sD;
	QString rw_sD2;
	QString rw_sD3;
//	QString rw_sErrTit;				// error title
	QString rw_sErrTxt;				// error text
	QString rw_sRootDir;			// root directory holding the movie files - absolute path
	QString rw_sTag;				// a tag line from tags.txt, will be chopped during processing
//	QString sRwD3;
//	QString sRwD4;
	QString rw_sFilePathName;		// full path and name of tags.txt file

	QStringList rw_slD;
	QStringList rw_slGenres;
	QStringList rw_slLangs;
	QStringList rw_slMovies;
	QStringList rw_slTags;
	QStringList rw_slTags2;

	StrMovie rw_strMovieClear;			// movie struct entity to clear the struct entity
	StrMovie rw_strMovieD;				// movie struct entity
	StrMovie rw_strMovieD2;				// movie struct entity
	QList<StrMovie> rw_liMoviesStr;		// list with movie structs for movies table widget
	QList<StrMovie> rw_liMoviesStrTags;	// list with movie structs from tags.txt file

	StrTagsLists rw_strTagsListsD;		// tags lists struct entity - 3 tag lists: genre, language and tags

// ***********************************************************
// **  Dir functions										**
// ***********************************************************
int rwDirStatus(const QString& rw_sD)				// check dir status: 0=does not exist, 100=dir, +10=readable, +1=writable
{
	QFileInfo qfi(rw_sD);
	if (qfi.exists() == false) {				// 0 = does not exist
		rw_iD = 0;
		return rw_iD;
	}

    if (qfi.isDir() == true) {					// 100 = is dir
        rw_iD = 100;
    }

    if (qfi.isReadable() == true) {				// +10 = is readable
        rw_iD += 10;
	}

    if (qfi.isWritable() == true) {				// +1 = is writable
        rw_iD += 1;
	}

	return rw_iD;
}


// ***********************************************************
// **  file functions										**
// ***********************************************************
int rwFileBackupCreate(const QString& rw_sD)		// create a backup before writing the current file (replacing existing backup),
													// -1=undefined, 0=OK, 11=does not exist, 21=not a file, 31=not writable, 41=bak2 still exist=previous error, 51=bak to bak2 failed, 61=file to bak failed, 71=bak2 to bak failed, 81=del bak2 failed
{
	rw_sFilePathName = rw_sD;
//	iRwCB = -1;											// return from create backup: -1=undefined, 0=OK, 11=not writable, 21=bak2 still exist=previous error, 31=bak to bak2 failed, 41=file to bak failed, 51=bak2 to bak failed, 61=del bak2 failed
	rw_iBak = -1;										// -1=undefined, 0=could not be created (no original file), 1=exists, 2=just newly created, 3=try to create from *.bak2
	rw_iBak2 = -1;										// -1=undefined, 0=could not be created (no *.bak file), 1=exists, 2=just newly created

	rw_iFs2 = rwFileStatus(rw_sFilePathName);
	// *****  step 1)  only proceed, if we already have a writable tags.txt file
	if (rw_iFs2 > 0) {									// file exists - backup needed or 'is dir' = error
		if (rw_iFs2 < 100) {
			return 21;									// not a file
		} else if (rw_iFs2 != 111 and rw_iFs2 != 101) {
			return 31;									// file not writable - abort
		}
	} else {											// no file, no backup needed
		return 11;
	}

	// *****  step 2)  if *.bak2 exists - abort, we failed previously, manual interaction required
	if (rwFileStatus(rw_sFilePathName + ".bak2") > 0) {	// *.bak2 still exists, we failed previously, manual interaction required, abort
		return 41;
	}

	// *****  step 3)  if *.bak exists, rename to *.bak2
	if (rwFileStatus(rw_sFilePathName + ".bak") > 0) {	// backup exists, try to re-name
		rw_iBak = 1;
		if (rwFileReName(rw_sFilePathName + ".bak", rw_sFilePathName + ".bak2") == true) {
			rw_iBak2 = 2;
		} else {										// error step 2)
			return 51;
		}
	} else {											// no backup file (yet) continue
		rw_iBak = 0;
		rw_iBak2 = 0;
	}

	// *****  step 4) rename old file to *.bak, we already know, that we have a writable file
	if (rwFileReName(rw_sFilePathName, rw_sFilePathName + ".bak") == false) {	// creating backup failed
		if (rw_iBak2 == 2) {							// try to rename just previously created *.bak2 back to *.bak
			rw_iBak = 3;
			if (rwFileReName(rw_sFilePathName + ".bak2", rw_sFilePathName + ".bak") == false) {
				return 71;									// leave the *.bak2 file untouched for now - manual intervention needed
			} else {
				rw_iBak = 2;									// successfully created *.bak from *.bak2 - *.bak2 can be deleted
			}
		} else {
			return 61;
		}
	} else {
//		bRwCont = true;
	}

/*	now integrated into step 4
	// *****  step 5) rename *.bak2 back to *.bak, if we could not create *.bak from file
	if (iRwBak == 3) {
		if (rwCommonFileReName(sRwD2 + ".bak2", sRwD2 + ".bak") == false) {
			return 61;									// leave the *.bak2 file untouched for now - manual intervention needed
		} else {
			iRwBak = 2;									// successfully created *.bak from *.bak2 - *.bak2 can be deleted
		}
	} else {
		bRwCont = true;
	}
*/
	// *****  step 6) del *.bak2 file
	rw_iD = rwFileStatus(rw_sFilePathName + ".bak2");				// check for existing file
	if ((rw_iD == 111 or rw_iD == 101) and rw_iBak != 3) {	// *.bak2 exists and can be deleted (*.bak was re-created)
		if (rwFileDelete(rw_sFilePathName + ".bak2") == false) {
			return 81;
		} else {
//			bRwCont = true;
		}
	}

	return 0;
}

QString rwFileBackupErrorTxt(int rw_iD, const QString& rw_sD)	// return a human readable error message from given error code, -110 from rwBackupCreate, -xxx from rwDomainFileSave
{
	rw_sFilePathName = rw_sD;
	switch (rw_iD) {
		case 0:
			rw_sErrTxt = QObject::tr("The error is, that this is _NOT_ an error!\nSorry, this must never happen!");
			break;
		case 11:
		    rw_sErrTxt = QObject::tr("The file\n"
			                        "%1\n"
			                        "does not exist!\n"
			                        "No need for a backup.\n"
			                        "This should never happen!").arg(rw_sFilePathName);
			break;
		case 21:
		    rw_sErrTxt = QObject::tr("%1\n"
			                        "is not a file!").arg(rw_sFilePathName);
			break;
		case 31:
		    rw_sErrTxt = QObject::tr("The existing file\n"
			                        "%1\n"
			                        "is not writable!\n").arg(rw_sFilePathName);
			break;
/*		case 41:
		    rw_sErrTxt = QObject::tr("A *.bak version of\n"
			                        "%1\n"
			                        "exists without file.\n"
			                        "This indicates a previous error.\n"
			                        "Manual interaction is now required!").arg(rw_sFilePathName);
			break;*/
		case 41:
		    rw_sErrTxt = QObject::tr("A *.bak2 version (2nd backup) of\n"
			                        "%1\n"
			                        "still exists.\n"
			                        "This indicates a previous error.\n"
			                        "Manual interaction is required!").arg(rw_sFilePathName);
			break;
		case 51:
		    rw_sErrTxt = QObject::tr("Temporary renaming the existing backup\n"
			                        "%1.bak\n"
			                        "to *.bak2 failed!").arg(rw_sFilePathName);
			break;
		case 61:
		    rw_sErrTxt = QObject::tr("Creating a backup from\n"
			                        "%1\n"
			                        "failed!").arg(rw_sFilePathName);
			break;
		case 71:
		    rw_sErrTxt = QObject::tr("Creating a backup from\n"
			                        "%1\n"
			                        "failed!\n"
			                        "Renaming\n"
			                        "%1.bak2\n"
			                        "back to *.bak failed also!\n"
			                        "Manual interaction is now required!").arg(rw_sFilePathName);
			break;
		case 81:
		    rw_sErrTxt = QObject::tr("Deleting the temporary 2nd backup\n"
			                        "%1.bak2\n"
			                        "failed!\n"
			                        "Manual interaction is now required!").arg(rw_sFilePathName);
			break;
		case 91:
		    rw_sErrTxt = QObject::tr("Re-creating\n"
			                        "%1\n"
			                        "from backup *.bak failed!\n"
			                        "Manual interaction is now required!").arg(rw_sFilePathName);
			break;
		default:
		    rw_sErrTxt = QObject::tr("Undefined error!\n"
			                        "Sorry, this must never happen!\n"
			                        "Aborting (hopefully)...");		// also covers -1 case
	}
	return rw_sErrTxt;
}

bool rwFileDelete(const QString& rw_sD)				// delete file
{
	QFile f(rw_sD);
	return (f.remove());
}

bool rwFileReName(const QString& rw_sD, const QString& rw_sD2)	// rename file type: rw_sD=old, rw_sD2=new
{
	QFile f(rw_sD);
	return (f.rename(rw_sD2));
}

int rwFileStatus(const QString& rw_sD)				// check file status: 0=does not exist, 100=file, +10=readable, +1=writable
{
    QFileInfo qfi(rw_sD);
    if (qfi.exists() == false) {						// 0 = does not exist
        rw_iD = 0;
        return rw_iD;
    }

    if (qfi.isFile() == true) {							// 100 = is file
        rw_iD = 100;
    }

    if (qfi.isReadable() == true) {						// +10 = is readable
        rw_iD += 10;
    }

    if (qfi.isWritable() == true) {						// +1 = is writable
        rw_iD += 1;
    }

    return rw_iD;
}


StrMovie rwMovieStrClear()							// clear movie struct
{
	rw_strMovieClear.sPath = "";
	rw_strMovieClear.sName = "";
	rw_strMovieClear.sSuffix = "";
	rw_strMovieClear.sGenre = "";
	rw_strMovieClear.sLang1 = "";
	rw_strMovieClear.sLang2 = "";
	rw_strMovieClear.sTag1 = "";
	rw_strMovieClear.sTag2 = "";
	rw_strMovieClear.sTag3 = "";
	rw_strMovieClear.sTag4 = "";
	rw_strMovieClear.fSize = 0;
	return rw_strMovieClear;
}


// ***********************************************************
// **  movie functions										**
// ***********************************************************
QStringList rwMoviesFileListGet(const QString& rw_sD)	// get list of movie files in root directory - rootdir
{
    QDir qdir(rw_sD);
        qdir.setFilter(QDir::Files | QDir::Readable);
        rw_slD.clear();
        rw_slD << "*.avi" << "*.mkv" << "*.mp4" << "*.mpeg" << "*.mpg" << "*.ts" << "*.vdr" << "*.webm";
        qdir.setNameFilters(rw_slD);
        qdir.setSorting(QDir::Name);
    return qdir.entryList();
}

QList<StrMovie> rwMoviesListStrGet(const QString& rw_sD)	// get list with movie struct entities, join movie list with found tags - rootdir
{
	rw_sRootDir = rw_sD;

	rw_slMovies = rwMoviesFileListGet(rw_sRootDir);				// rw_slMovies only contains name.suffix, not the path
//cout << "rwMoviesStrListGet: rw_slMovies count = " << QString::number(rw_slMovies.count()).toStdString() << endl;                                    // *****  debug
	rw_liMoviesStrTags = rwTagsListStrGet(rw_sRootDir);			// rw_liMoviesStrTags contains path, name, suffix
//cout << "rwMoviesListStrGet: rw_liMoviesStrTags count = " << QString::number(rw_liMoviesStrTags.count()).toStdString() << endl;                                    // *****  debug

	rw_liMoviesStr.clear();
	if (rw_slMovies.count() > 0) {
		for (rw_iD = 0; rw_iD < rw_slMovies.count(); ++rw_iD) {	// iterate over the movie file list
			rw_sD2 = rw_slMovies.at(rw_iD);
			rw_strMovieD = rwMovieStrClear();
			rw_strMovieD.sPath = rw_sRootDir;			// path, ends on '/'
			rw_iD2 = rw_sD2.lastIndexOf(".");				// the filter shows only files with at least one '.'
			rw_strMovieD.sName = rw_sD2.left(rw_iD2);	// name
			rw_sD2.remove(0, rw_iD2 + 1);
			rw_strMovieD.sSuffix = rw_sD2;				// suffix

			rw_sD3 = rw_strMovieD.sPath + rw_strMovieD.sName + "." + rw_strMovieD.sSuffix;	// path already ends on '/'
			QFileInfo qfi(rw_sD3);
				if (qfi.exists() == false) {				// -1 = does not exist
					rw_fD = -1;
				} else {
					rw_fD = qfi.size();
					rw_fD = rw_fD/1024/1024/1024;
				}
			rw_strMovieD.fSize = rw_fD;					// file size
//cout << "rwMoviesListStrGet: rw_strMovieD.fSize = " << QString::number(rw_fD).toStdString() << endl;                                    // *****  debug

			if (rw_liMoviesStrTags.count() > 0) {
				for (rw_iD2 = 0; rw_iD2 < rw_liMoviesStrTags.count(); ++rw_iD2) {	// iterate over the tag list file for each movie file
//cout << "rwMoviesListStrGet: entering iteration over tag list " << endl;                                    // *****  debug
					rw_strMovieD2 = rwMovieStrClear();
					rw_strMovieD2 = rw_liMoviesStrTags.at(rw_iD2);
					rw_sD2 = rw_strMovieD2.sPath + rw_strMovieD2.sName + "." + rw_strMovieD2.sSuffix;

					if (rw_sD3 == rw_sD2) {	// match, add tags to found movies, replace movie with the tags file version
						rw_strMovieD.sGenre = rw_strMovieD2.sGenre;
						rw_strMovieD.sLang1 = rw_strMovieD2.sLang1;
						rw_strMovieD.sLang2 = rw_strMovieD2.sLang2;
						rw_strMovieD.sTag1 = rw_strMovieD2.sTag1;
						rw_strMovieD.sTag2 = rw_strMovieD2.sTag2;
						rw_strMovieD.sTag3 = rw_strMovieD2.sTag3;
						rw_strMovieD.sTag4 = rw_strMovieD2.sTag4;
						break;
					}
				}
			}
			rw_liMoviesStr.append(rw_strMovieD);
		}
	}
	return rw_liMoviesStr;
}


// ***********************************************************
// **  Tag file functions									**
// ***********************************************************
QStringList rwTagsFileRead(const QString& rw_sD)	// read tag file tags.txt with one tag per line, tags only, skip '#' lines - root dir given
{
	rw_sRootDir = rwTagsRootDirFormat(rw_sD);
	rw_slTags.clear();

	QFile qf(rw_sRootDir + "tags.txt");
	if (!qf.open(QIODevice::ReadOnly | QIODevice::Text)) {	// if we cannot open the file for reading, abort
		rw_slTags.append("filmat11");
		rw_slTags.append("Error opening file");
	} else {
		QTextStream in(&qf);							// initiate the stream to read the file line by line
		while (!in.atEnd()) {							// read the file (textstream)
			rw_sD2 = in.readLine();							// read line by line
			if (rw_sD2.left(1) != "#") {					// skip '#' comments
				rw_slTags.append(rw_sD2);
			}
		}
		qf.close();									// close channel list file at EOF
	}
//cout << "rwTagsFileRead - first line: " << rw_slTags.at(0).toStdString() << endl;                                               // *****  debug  *****
	return rw_slTags;
}

QStringList rwTagsFileSave(const QString& rw_sD, const QList<StrMovie>& rw_liMoviesStr)	// prepare saving tags.txt file - root dir, list of tag structs
{
//cout << "rwTagsFileSave - reached" << QString::number(rw_liStrMovies.count()).toStdString() << endl;                                               // *****  debug  *****
	rw_sRootDir = rwTagsRootDirFormat(rw_sD);			// root dir
	rw_sFilePathName = rw_sRootDir + "tags.txt";
	rw_liMoviesStrTags = rw_liMoviesStr;				// list with tag structs
	rw_slD.clear();

	if (rw_liMoviesStr.count() == 0) {					// empty movie list
		rw_slD.append("99");
		rw_slD.append(QObject::tr("Information!\nThe movie list is empty.\nNothing to do."));
		return rw_slD;
	}

	rw_iFs = rwFileStatus(rw_sFilePathName);
	if (rw_iFs > 0) {									// target exists already - action required
		if (rw_iFs == 111 or rw_iFs == 101) {			// 101 = writable but not readable - oh well !? - create backup
				rw_iD2 = rwFileBackupCreate(rw_sFilePathName);
				if (rw_iD2 != 0) {							// we had errors while creating the backup > abort
					rw_slD.append("99");
					rw_slD.append(rwFileBackupErrorTxt(rw_iD2, rw_sFilePathName));
					return rw_slD;
				}
		} else {										// dir or not writable
			rw_slD.append("99");
			rw_slD.append(QObject::tr("Error!\n%1\nis either a directory or not writable!\n"
									 "Aborting!").arg(rw_sFilePathName));
			return rw_slD;
		}
	}

	// write tags.txt file
	if (rwTagsFileWrite(rw_sFilePathName, rw_liMoviesStrTags) == true) {	// success
		rw_slD.append("0");
		rw_slD.append(QObject::tr("Tag file\n%1\nsaved successfully.").arg(rw_sFilePathName));
	} else {											// error writing tags.txt
		rw_slD.append("99");
		rw_sD2 = QObject::tr("Writing the tag file\n%1\nfailed!").arg(rw_sFilePathName);
		if (rw_iFs > 0) {													// we created a backup! try to re-create old file from *.bak
			if (rwFileReName(rw_sFilePathName + ".bak",rw_sFilePathName) == false) {	// *.bak to file failed
				rw_sD2 = rw_sD2 + "\n" + rwFileBackupErrorTxt(81, rw_sFilePathName);	// error re-creating old tags.txt file from *.bak
		    } else {
			    rw_sD2 = rw_sD2 + "\n" + QObject::tr("Successfully re-created the old file\n%1\nfrom backup.").arg(rw_sFilePathName);	// error re-creating old tags.txt file from *.bak
		    }
		}
		rw_slD.append(rw_sD2);
	}
	return rw_slD;
}

bool rwTagsFileWrite(const QString& rw_sD, const QList<StrMovie>& rw_liMoviesStr)	// write tags.txt tag file - tag file full path + name, list of tag structs
{
//cout << "rwTagsFileWrite - reached" << QString::number(rw_liStrMovies.count()).toStdString() << endl;                                               // *****  debug  *****
	QFile file(rw_sD);
	if (file.open(QIODevice::WriteOnly)) {
		QTextStream stream(&file);

		stream << "# This is a tag list file from filmat11," << "\n";						// create file header
		stream << "# a small program to show movies via mpv." << "\n";
		stream << "# These tags, one line per movie, can be used to filter the movies." << "\n";
		stream << "# File format V01." << "\n";
		stream << "# Please do not edit manually. filmat11's import function is not overly robust! :-)" << "\n";
		stream << "#" << "\n";

		for (rw_iD = 0; rw_iD < rw_liMoviesStr.count(); ++rw_iD) {
			rw_strMovieD = rw_liMoviesStr.at(rw_iD);
//			rw_sD2 = QString::number(rw_strMovieD.fSize);
			stream << rw_strMovieD.sPath << "|" << rw_strMovieD.sName << "|" << rw_strMovieD.sSuffix << "|" << rw_strMovieD.sGenre << "|" << rw_strMovieD.sLang1 << "|" << rw_strMovieD.sLang2 << "|" << rw_strMovieD.sTag1 << "|" << rw_strMovieD.sTag2 << "|" << rw_strMovieD.sTag3 << "|" << rw_strMovieD.sTag4 << "|" << QString::number(rw_strMovieD.fSize, 'f', 2) << "\n"; // add entry
		}
		file.close();
		return true;
	} else {
		return false;
	}
}

StrTagsLists rwTagsListsGet(const QList<StrMovie>& rw_liStrMovies)	// get 3 lists with genre, language and tags tags - movie struct list
{
	rw_slGenres.clear();
	rw_slGenres.append("");
	rw_slLangs.clear();
	rw_slLangs.append("");
	rw_slTags.clear();
	rw_slTags.append("");
	rw_strTagsListsD.slGenres = rw_slGenres;
	rw_strTagsListsD.slLangs = rw_slLangs;
	rw_strTagsListsD.slTags = rw_slTags;

	if (rw_liStrMovies.count() > 0) {
		for (rw_iD = 0; rw_iD < rw_liStrMovies.count(); ++rw_iD) {
			rw_strMovieD = rw_liStrMovies.at(rw_iD);
			if (rw_strMovieD.sGenre > "") {
				rw_slGenres.append(rw_strMovieD.sGenre);
			}
			
			if (rw_strMovieD.sLang1 > "") {
				rw_slLangs.append(rw_strMovieD.sLang1);
			}
			if (rw_strMovieD.sLang2 > "") {
				rw_slLangs.append(rw_strMovieD.sLang2);
			}
			
			if (rw_strMovieD.sTag1 > "") {
				rw_slTags.append(rw_strMovieD.sTag1);
			}
			if (rw_strMovieD.sTag2 > "") {
				rw_slTags.append(rw_strMovieD.sTag2);
			}
			if (rw_strMovieD.sTag3 > "") {
				rw_slTags.append(rw_strMovieD.sTag3);
			}
			if (rw_strMovieD.sTag4 > "") {
				rw_slTags.append(rw_strMovieD.sTag4);
			}
		}
		rw_slGenres.sort();
		rw_slGenres.removeDuplicates();
		rw_slLangs.sort();
		rw_slLangs.removeDuplicates();
		rw_slTags.sort();
		rw_slTags.removeDuplicates();
		rw_strTagsListsD.slGenres = rw_slGenres;
		rw_strTagsListsD.slLangs = rw_slLangs;
		rw_strTagsListsD.slTags = rw_slTags;
	}
	return rw_strTagsListsD;
}

QList<StrMovie> rwTagsListStrGet(const QString& rw_sD)	// get list with tags struct entities - rootdir
{
	rw_slTags2 = rwTagsFileRead(rw_sD);
	rw_liMoviesStrTags.clear();
	if (rw_slTags2.count() > 0 and rw_slTags2.at(0) != "filmat11") {
		for (rw_iD = 0; rw_iD < rw_slTags2.count(); ++rw_iD) {	// iterate over just loaded tags.txt file list and extract tags per movie entry - 1 line = tags for 1 movie
			rw_sTag = rw_slTags2.at(rw_iD);
			if (rw_sTag.length() > 0) {							// skip empty lines - possible ?
				rw_iD2 = rw_sD2.count("|");
				if (rw_iD2 == 10) {								// we expect 10 seperators, looks like a valid entry...
					rw_iD3 = rw_sD2.indexOf("|");					// path to movie
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sPath = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sPath = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// movie name
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sName = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sName = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// movie suffix
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sSuffix = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sSuffix = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// genre
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sGenre = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sGenre = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// lang 1
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sLang1 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sLang1 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// lang 2
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sLang2 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sLang2 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// tag 1
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sTag1 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sTag1 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// tag 2
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sTag2 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sTag2 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// tag 3
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sTag3 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sTag3 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					rw_iD3 = rw_sTag.indexOf("|");				// tag 4
					if (rw_iD3 == 0) {								// '|' is first = empty field
						rw_strMovieD.sTag4 = "";
						rw_sTag.remove(0,1);
					} else {
						rw_strMovieD.sTag4 = rw_sTag.left(rw_iD3);
						rw_sTag.remove(0,rw_iD3 + 1);	
					}

					if (rw_sTag.length() > 0) {					// size in GB
						rw_strMovieD.fSize = rw_sTag.toFloat();		// need error handling here ! *****************************
					} else {
						rw_strMovieD.fSize = 0;						// last field empty
					}

					rw_liMoviesStrTags.append(rw_strMovieD);	// add tags struct to tag struct list
				} else {										// invalid entry
				}
			}
		} 
	}
	return rw_liMoviesStrTags;
} 

QString rwTagsRootDirFormat(const QString& rw_sD)   // ensure, that root dir ends with a '/'
{
	rw_sRootDir = rw_sD;
	if (rw_sRootDir.right(1) != "/") {
		rw_sRootDir = rw_sRootDir + "/";
	}
	return rw_sRootDir;
}
