/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2021-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/

#ifndef _RW_H_
#define _RW_H_

#include <QDir>
#include <QFile>
#include <QFileInfo>
#include <QList>
#include <QObject>
#include <QString>
#include <QStringList>
#include <QTextStream>

#include "structs.h"

// ******  debug  *****
#include <iostream>
using namespace std;


    int rwDirStatus(const QString& rw_sD);                       	// check dir status: 0=does not exist, 100=dir, +10=readable, +1=writable

	int rwFileBackupCreate(const QString& rw_sD2);					// create a backup before writing the current file (overwrite existing backup), -1=undefined, 0=OK, 11=not writable, 21=bak2 still exist=previous error, 31=bak to bak2 failed, 41=file to bak failed, 51=bak2 to bak failed, 61=del bak2 failed
	QString rwFileBackupErrorTxt(int rw_iD, const QString& rw_sD2);	// return a human readable error message from given error code
	bool rwFileDelete(const QString& rw_sD);						// delete file
	bool rwFileReName(const QString& rw_sD, const QString& rw_sD2);	// rename file type: rw_sD=old, rw_sD2=new
    int rwFileStatus(const QString& rw_sD);                         // check file status: 0=does not exist, 100=dir, +10=readable, +1=writable

    StrMovie rwMovieStrClear();										// clear movie struct

    QStringList rwMoviesFileListGet(const QString& rw_sD);          // get list of readable files in given directory
    QList<StrMovie> rwMoviesListStrGet(const QString& rw_sD);		// get list with movie struct entities, join movie list with found tags - rootdir

    QStringList rwTagsFileRead(const QString& rw_sD);               							// read tag file tags.txt with one tag per line, tags only, skip '#' lines - root dir given
    QStringList rwTagsFileSave(const QString& rw_sD, const QList<StrMovie>& rw_liStrMovies);	// prepare saving tags.txt file - root dir, list of tag structs
    bool rwTagsFileWrite(const QString& rw_sD, const QList<StrMovie>& rw_liStrMovies);			// write tags.txt tag file - root dir, list of tag structs
    StrTagsLists rwTagsListsGet(const QList<StrMovie>& rw_liStrMovies);					// get 3 lists with genre, language and tags tags
    QList<StrMovie> rwTagsListStrGet(const QString& rw_sD);										// get list with tags struct entities - rootdir
    QString rwTagsRootDirFormat(const QString& rw_sD);               							// ensure, that root dir ends with a '/'

#endif
