/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2021-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *  settings (configuration) dialog                                  *
 *********************************************************************/

#include "settings.h"

SetDlg::SetDlg(QDialog *parent)
		: QDialog(parent)
{
	uiDlgSet.setupUi(&wDlgSet);					// settings dialog
}

SetDlg::~SetDlg()
{
}


// ****************************************************************
// **  settings read/write                                       **
// ****************************************************************
void SetDlg::settingsRead()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		wDlgSet.resize(settings.value("size", QSize(400, 400)).toSize());
		wDlgSet.move(settings.value("pos", QPoint(200, 200)).toPoint());

        uiDlgSet.dlgSet_l_rootDir->setText(settings.value("rootDir", "").toString());      // path to root directory with movies
		
	settings.endGroup();
}

void SetDlg::settingsWrite()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		settings.setValue("rootDir", uiDlgSet.dlgSet_l_rootDir->text());
	settings.endGroup();

	settings.sync();
}

void SetDlg::settingsWritePos()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		settings.setValue("pos", wDlgSet.pos());
		settings.setValue("size", wDlgSet.size());
	settings.endGroup();

	settings.sync();
}


// ****************************************************************
// **  save button disable/enable                                **
// ****************************************************************
void SetDlg::saveEnable(bool sd_bD)					// enable=true / disable=false save button
{
	sd_bD2 = sd_bD;
	uiDlgSet.dlgSet_pb_Save->setEnabled(sd_bD);
}


// ****************************************************************
// **  settings dialog open/close                                **
// ****************************************************************
void SetDlg::setDlgShow()							// button 'settings' clicked, show settings dialog
{
	settingsRead();

	uiDlgSet.dlgSet_pb_setRootDir->setIcon(QIcon("/usr/share/filmat11/icons/scalable/open.svg"));

//	saveDisable();										// disable save button on first open

	connect(uiDlgSet.dlgSet_pb_Save, SIGNAL(clicked()), this, SLOT(clickedSave()));
	connect(uiDlgSet.dlgSet_pb_setRootDir, SIGNAL(clicked()), this, SLOT(rootDirDlgClicked()));
	connect(uiDlgSet.dlgSet_pb_Cancel, SIGNAL(clicked()), this, SLOT(clickedClose()));

	sd_bD = false;
	sd_bD2 = false;

	wDlgSet.show();
}

void SetDlg::setDlgClose()							// close settings dialog
{
	settingsWritePos();
	emit sigSetDlgClose(sd_bD2);
	wDlgSet.hide();
}


// ****************************************************************
// **  handle button clicked                                     **
// ****************************************************************
void SetDlg::clickedClose()							// close clicked in settings dialog
{
	setDlgClose();
}

void SetDlg::clickedSave()							// save clicked in settings dialog
{
	settingsWrite();
//	saveDisable();
	clickedClose();
}

void SetDlg::rootDirDlgClicked()					// button to set root directory clicked
{
    sd_sRootDir = uiDlgSet.dlgSet_l_rootDir->text();
	rootDirChk(sd_sRootDir);

	sd_sRootDirSelected = QFileDialog::getExistingDirectory(this, tr("Select filmat11's start directory"),
														sd_sRootDir,
														QFileDialog::ShowDirsOnly);						// | QFileDialog::DontResolveSymlinks);
	if (sd_sRootDirSelected > "" and sd_sRootDirSelected != sd_sRootDir) {
		if (rootDirChk(sd_sRootDirSelected) == true) {
			sd_sRootDir = sd_sRootDirSelected;
			rootDirSet();
		}	
	}
}

void SetDlg::rootDirSet()							// set the programs root directory
{
	if (rootDirChk(sd_sRootDir) == true) {			// double check?
		uiDlgSet.dlgSet_l_rootDir->setText(sd_sRootDir);
	}
//cout << "rootDirSet after setText " << endl;
	
	saveEnable(true);
}

bool SetDlg::rootDirChk(const QString &sd_sD)		// check if root directory is valid
{
	QFileInfo finf(sd_sRootDir);
	if (finf.isDir() == false or finf.isReadable() == false ) {
		QMessageBox::information(this, tr("filmat11 information - directory invalid !"), 
									   tr("Attention, the entered/selected directory is invalid.\n"
										  "It either does not exist or it is not readable.\n"
										  "The directory will either be set to your home directory or\n"
										  "if changed, the newly selected directory will be ignored."));
		sd_sRootDir = QDir::homePath();																// invalid directory - set root dir to user home path
		return false;
	}
	return true;
}

