/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2021-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *  edit movie tags dialog											 *
 *********************************************************************/

#include "tags.h"

TagDlg::TagDlg(QDialog* parent)
		: QDialog(parent)
{
	uiDlgTag.setupUi(&wDlgTag);							// edit tag dialog

	connect(uiDlgTag.dlgTags_pb_Cancel, SIGNAL(clicked()), this, SLOT(td_ClickedClose()));
	connect(uiDlgTag.dlgTags_pb_Save, SIGNAL(clicked()), this, SLOT(td_ClickedSave()));
}

TagDlg::~TagDlg()
{
}


// ****************************************************************
// **  settings read/write                                       **
// ****************************************************************
/*
void TagDlg::settingsRead()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		wDlgSet.resize(settings.value("size", QSize(400, 400)).toSize());
		wDlgSet.move(settings.value("pos", QPoint(200, 200)).toPoint());

        uiDlgSet.dlgSet_l_rootDir->setText(settings.value("rootDir", "").toString());      // path to root directory with movies
		
	settings.endGroup();
}

void TagDlg::settingsWrite()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		settings.setValue("rootDir", uiDlgSet.dlgSet_l_rootDir->text());
	settings.endGroup();

	settings.sync();
}

void TagDlg::settingsWritePos()
{
	QSettings settings("filmat11", "filmat11");

	settings.beginGroup("settings");
		settings.setValue("pos", wDlgSet.pos());
		settings.setValue("size", wDlgSet.size());
	settings.endGroup();

	settings.sync();
}
*/


// ****************************************************************
// **  save button disable/enable                                **
// ****************************************************************
void TagDlg::td_SaveEnable(bool td_bD)				// enable=true / disable=false save button
{
	td_bLiMoviesStrChgd = td_bD;
	uiDlgTag.dlgTags_pb_Save->setEnabled(td_bLiMoviesStrChgd);
	uiDlgTag.dlgTags_pb_Save->setFocus(Qt::TabFocusReason);
}


// ****************************************************************
// **  tag dialog open/close                                **
// ****************************************************************
void TagDlg::td_Show(QList<StrMovie> td_liMoviesStrD, int td_iD)	// show edit tag dialog - movie str list, current movie index
{
//cout << "tagDlgShow() - reached " << endl;                               // *****  debug  *****
	td_iMovieCurIdx = td_iD;
	td_liMoviesStr = td_liMoviesStrD;
	td_strMovieIn = td_liMoviesStr.at(td_iMovieCurIdx);
	td_bLiMoviesStrChgd = false;
//	settingsRead();

	td_SaveEnable(false);							// disable save button on first open
	td_ListsTagsGen();
	td_CbsPop();
	td_Pop();

	wDlgTag.show();
}

void TagDlg::td_Close()								// close edit tag dialog
{
//	settingsWritePos();
	if (td_bLiMoviesStrChgd == true) {
		td_strMovieD = td_strMovieIn;
		td_strMovieD.sGenre = uiDlgTag.dlgTags_cb_Genre->currentText();
		td_strMovieD.sLang1 = uiDlgTag.dlgTags_cb_Lang1->currentText();
		td_strMovieD.sLang2 = uiDlgTag.dlgTags_cb_Lang2->currentText();
		td_strMovieD.sTag1 = uiDlgTag.dlgTags_cb_Tag1->currentText();
		td_strMovieD.sTag2 = uiDlgTag.dlgTags_cb_Tag2->currentText();
		td_strMovieD.sTag3 = uiDlgTag.dlgTags_cb_Tag3->currentText();
		td_strMovieD.sTag4 = uiDlgTag.dlgTags_cb_Tag4->currentText();
	}
	emit sigTagDlgClose(td_bLiMoviesStrChgd, td_liMoviesStr);
	wDlgTag.hide();
}


void TagDlg::td_CbsCon(bool td_bD)					// connect (true) / disconnect (false) comboboxes signals
{
	if (td_bD == true) {
		td_GenreCbCon(true);
		td_LangCbsCon(true);
		td_TagCbsCon(true);
	} else {
		td_GenreCbCon(false);
		td_LangCbsCon(false);
		td_TagCbsCon(false);
	}
}

void TagDlg::td_CbsPop()							// populate comboboxes
{
	td_GenreCbPop();
	td_LangCbsPop();
	td_TagCbsPop();
}


// ****************************************************************
// **  handle button clicked                                     **
// ****************************************************************
void TagDlg::td_ClickedClose()						// close clicked in edit tag dialog
{
	td_bLiMoviesStrChgd = false;						// don't do anything
	td_Close();
}

void TagDlg::td_ClickedDelTag()						// delete a Tag entry (genre, language or tag) clicked - show popup to select what to delete
{
	
}

void TagDlg::td_ClickedSave()						// save clicked in edit tag dialog
{
	td_bLiMoviesStrChgd = true;
	td_strMovieD = td_strMovieIn;
		td_strMovieD.sGenre = uiDlgTag.dlgTags_cb_Genre->currentText();
//cout << "tagDlgClickedSave() , genre = " << td_strMovieD.sGenre.toStdString() << endl;
	
		td_strMovieD.sLang1 = uiDlgTag.dlgTags_cb_Lang1->currentText();
		td_strMovieD.sLang2 = uiDlgTag.dlgTags_cb_Lang2->currentText();
	
		td_strMovieD.sTag1 = uiDlgTag.dlgTags_cb_Tag1->currentText();
		td_strMovieD.sTag2 = uiDlgTag.dlgTags_cb_Tag2->currentText();
		td_strMovieD.sTag3 = uiDlgTag.dlgTags_cb_Tag3->currentText();
		td_strMovieD.sTag4 = uiDlgTag.dlgTags_cb_Tag4->currentText();
	
	td_liMoviesStr.replace(td_iMovieCurIdx, td_strMovieD);
//	settingsWrite();
//	saveDisable();
	td_Close();
}


//  **********  genre related  **********
void TagDlg::td_GenreCbCon(bool td_bD)				// connect (true) / disconnect (false) comboboxes signals
{
	if (td_bD == true) {
		connect(uiDlgTag.dlgTags_cb_Genre, SIGNAL(currentIndexChanged(int)), this, SLOT(td_GenreChgd(int)));
		connect(uiDlgTag.dlgTags_cb_Genre->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_GenreCbTextChgd()));
	} else {
		disconnect(uiDlgTag.dlgTags_cb_Genre, SIGNAL(currentIndexChanged(int)), this, SLOT(td_GenreChgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Genre->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_GenreCbTextChgd()));
	}
}

void TagDlg::td_GenreCbPop()						// populate genre combobox
{
//cout << "tagDlgGenreCbPop() - reached " << endl;                               // *****  debug  *****
	td_GenreCbCon(false);
	uiDlgTag.dlgTags_cb_Genre->clear();
	uiDlgTag.dlgTags_cb_Genre->addItems(td_slGenres);
	td_GenreCbCon(true);
}

void TagDlg::td_GenreCbTextChgd()	// text in genre combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Genre->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Genre->setCurrentText(td_sTagNew);
	}
	td_GenreChgd(uiDlgTag.dlgTags_cb_Genre->currentIndex());
}

void TagDlg::td_GenreChgd(int td_iD)				// genre changed
{
	td_bLiMoviesStrChgd = true;
	td_SaveEnable(true);								// save button: enable=true / disable=false save button
}

bool TagDlg::td_InputTxtChk(const QString &td_sD)	// check input text against | , replace any occurance with _ , true = changes where required
{
	td_sD2 = td_sD;
	if (td_sD2.contains("|")) {
		QMessageBox::information(this, tr("filmat11 information - | replaced by _ !"), 
										tr("The character '|' cannot be used in a tag,\n"
										"as it is used as the field separator in the tag list file.\n"
										"It will automatically be replaced by '_' ."));
		td_sTagTxtCor = td_sD2.replace("|", "_");
		return true;
	}
	return false;
}

//  **********  language related  **********
void TagDlg::td_LangCbsCon(bool td_bD)				// connect (true) / disconnect (false) language combobox signals, no need to handle 1 and 2 separate ?
{
	if (td_bD == true) {
		connect(uiDlgTag.dlgTags_cb_Lang1, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Lang1Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Lang1->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Lang1CbTextChgd()));
		connect(uiDlgTag.dlgTags_cb_Lang2, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Lang2Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Lang2->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Lang2CbTextChgd()));
	} else {
		disconnect(uiDlgTag.dlgTags_cb_Lang1, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Lang1Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Lang1->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Lang1CbTextChgd()));
		disconnect(uiDlgTag.dlgTags_cb_Lang2, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Lang2Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Lang2->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Lang2CbTextChgd()));
	}
}

void TagDlg::td_LangCbsPop()						// populate lang1+2 comboboxes
{
	td_LangCbsCon(false);
	uiDlgTag.dlgTags_cb_Lang1->clear();
	uiDlgTag.dlgTags_cb_Lang1->addItems(td_slLangs);
	uiDlgTag.dlgTags_cb_Lang2->clear();
	uiDlgTag.dlgTags_cb_Lang2->addItems(td_slLangs);
	td_LangCbsCon(true);
}

void TagDlg::td_LangCbsPopRe()						// re-populate lang1+2 comboboxes
{
	td_sD = uiDlgTag.dlgTags_cb_Lang1->currentText();
	td_sD2 = uiDlgTag.dlgTags_cb_Lang2->currentText();
	td_LangCbsPop();
	uiDlgTag.dlgTags_cb_Lang1->setCurrentText(td_sD);
	uiDlgTag.dlgTags_cb_Lang2->setCurrentText(td_sD2);
}

void TagDlg::td_Lang1CbTextChgd()					// text in lang1 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Lang1->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Lang1->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_LangChkDbl() == 0) {		// valid language change
		if (td_strMovieIn.sLang1 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_LangListIsIn() == 1) {
				td_LangListAddTo();							// new language entry - add to comboboxes
				td_LangCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Lang1->setCurrentText("");
	}
}

void TagDlg::td_Lang2CbTextChgd()					// text in lang2 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Lang2->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Lang2->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_LangChkDbl() == 0) {		// valid language change
		if (td_strMovieIn.sLang2 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_LangListIsIn() == 1) {
				td_LangListAddTo();							// new language entry - add to comboboxes
				td_LangCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Lang2->setCurrentText("");
	}
}

int TagDlg::td_LangChkDbl()							// check language tag against double - lang 1 or 2
{
//	if (td_sTagNew != "") {
	if (uiDlgTag.dlgTags_cb_Lang1->currentText() !=  uiDlgTag.dlgTags_cb_Lang2->currentText()) {
		return 0;
	}
	QMessageBox::information(this, tr("filmat11 information - language already set !"), 
								   tr("The selected language is already set as a search tag.\n"
									  "Clearing this one."));
//	}
	return 1;
}

void TagDlg::td_LangListAddTo()						// add new language tag to list of language tags
{
	td_slLangs.append(td_sTagNew);
	td_slLangs.sort();
}

int TagDlg::td_LangListIsIn()						// check if new language tag already exists in list
{
	if (td_slLangs.contains(td_sTagNew) == true) {
		return 0;
	}
	return 1;
}

void TagDlg::td_Lang1Chgd(int td_iD)				// language 1 changed
{
	td_Lang1CbTextChgd();
}

void TagDlg::td_Lang2Chgd(int td_iD)				// language 2 changed
{
	td_Lang2CbTextChgd();
}	


void TagDlg::td_ListsTagsGen()						// generate lists with tags - genres, languages and tags
{
	td_strTagsListsD = rwTagsListsGet(td_liMoviesStr);
	td_slGenres = td_strTagsListsD.slGenres;
	td_slLangs = td_strTagsListsD.slLangs;
	td_slTags = td_strTagsListsD.slTags;
}


void TagDlg::td_StrMovieChgd()						// movie struct has changed, set corresponding bool to true and enable save button
{
	td_bLiMoviesStrChgd = true;
	td_SaveEnable(true);								// save button: enable=true / disable=false save button
}


//  **********  tag related  **********
void TagDlg::td_TagCbsCon(bool td_bD)				// connect (true) / disconnect (false) tags combobox signals, no need to handle 1 to 4 separate ?
{
	if (td_bD == true) {
		connect(uiDlgTag.dlgTags_cb_Tag1, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag1Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Tag1->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag1CbTextChgd()));
		connect(uiDlgTag.dlgTags_cb_Tag2, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag2Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Tag2->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag2CbTextChgd()));
		connect(uiDlgTag.dlgTags_cb_Tag3, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag3Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Tag3->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag3CbTextChgd()));
		connect(uiDlgTag.dlgTags_cb_Tag4, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag4Chgd(int)));
		connect(uiDlgTag.dlgTags_cb_Tag4->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag4CbTextChgd()));
	} else {
		disconnect(uiDlgTag.dlgTags_cb_Tag1, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag1Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Tag1->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag1CbTextChgd()));
		disconnect(uiDlgTag.dlgTags_cb_Tag2, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag2Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Tag2->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag2CbTextChgd()));
		disconnect(uiDlgTag.dlgTags_cb_Tag3, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag3Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Tag3->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag3CbTextChgd()));
		disconnect(uiDlgTag.dlgTags_cb_Tag4, SIGNAL(currentIndexChanged(int)), this, SLOT(td_Tag4Chgd(int)));
		disconnect(uiDlgTag.dlgTags_cb_Tag4->lineEdit(), SIGNAL(editingFinished()), this, SLOT(td_Tag4CbTextChgd()));
	}
}

void TagDlg::td_TagCbsPop()							// populate tag1-4 comboboxes
{
	td_TagCbsCon(false);
	uiDlgTag.dlgTags_cb_Tag1->clear();
	uiDlgTag.dlgTags_cb_Tag1->addItems(td_slTags);
	uiDlgTag.dlgTags_cb_Tag2->clear();
	uiDlgTag.dlgTags_cb_Tag2->addItems(td_slTags);
	uiDlgTag.dlgTags_cb_Tag3->clear();
	uiDlgTag.dlgTags_cb_Tag3->addItems(td_slTags);
	uiDlgTag.dlgTags_cb_Tag4->clear();
	uiDlgTag.dlgTags_cb_Tag4->addItems(td_slTags);
	td_TagCbsCon(true);
}

void TagDlg::td_TagCbsPopRe()						// re-populate tag 1-4 comboboxes
{
	td_sD = uiDlgTag.dlgTags_cb_Tag1->currentText();
	td_sD2 = uiDlgTag.dlgTags_cb_Tag2->currentText();
	td_sD3 = uiDlgTag.dlgTags_cb_Tag3->currentText();
	td_sD4 = uiDlgTag.dlgTags_cb_Tag4->currentText();
	td_TagCbsPop();
	uiDlgTag.dlgTags_cb_Tag1->setCurrentText(td_sD);
	uiDlgTag.dlgTags_cb_Tag2->setCurrentText(td_sD2);
	uiDlgTag.dlgTags_cb_Tag3->setCurrentText(td_sD3);
	uiDlgTag.dlgTags_cb_Tag4->setCurrentText(td_sD4);
}

void TagDlg::td_Tag1CbTextChgd()					// text in tag1 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Tag1->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Tag1->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_TagChkDbl(1) == 0) {		// valid tag change
		if (td_strMovieIn.sTag1 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_TagListIsIn() == 1) {
				td_TagListAddTo();							// new tag entry - add to comboboxes
				td_TagCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Tag1->setCurrentText("");
	}
}

void TagDlg::td_Tag2CbTextChgd()					// text in tag2 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Tag2->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Tag2->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_TagChkDbl(2) == 0) {		// valid tag change
		if (td_strMovieIn.sTag2 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_TagListIsIn() == 1) {
				td_TagListAddTo();							// new tag entry - add to comboboxes
				td_TagCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Tag2->setCurrentText("");
	}
}

void TagDlg::td_Tag3CbTextChgd()					// text in tag3 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Tag3->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Tag3->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_TagChkDbl(3) == 0) {		// valid tag change
		if (td_strMovieIn.sTag3 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_TagListIsIn() == 1) {
				td_TagListAddTo();							// new tag entry - add to comboboxes
				td_TagCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Tag3->setCurrentText("");
	}
}

void TagDlg::td_Tag4CbTextChgd()					// text in tag4 combobox has changed
{
	td_sTagNew = uiDlgTag.dlgTags_cb_Tag4->lineEdit()->text().trimmed();
	if (td_InputTxtChk(td_sTagNew) == true) {
		td_sTagNew = td_sTagTxtCor;
		uiDlgTag.dlgTags_cb_Tag4->setCurrentText(td_sTagNew);
	}
	if (td_sTagNew == "" or td_TagChkDbl(4) == 0) {		// valid tag change
		if (td_strMovieIn.sTag4 != td_sTagNew) {		// only enable save button, if entry was changed
				td_StrMovieChgd();							// set corresponding bool to true and enable save button
			if (td_TagListIsIn() == 1) {
				td_TagListAddTo();							// new tag entry - add to comboboxes
				td_TagCbsPopRe();							// re-populate comboboxes - show new entry also in other combobox
			}
		}
	} else {
		uiDlgTag.dlgTags_cb_Tag4->setCurrentText("");
	}
}

int TagDlg::td_TagChkDbl(int td_iD)					// check tag against double - changed combobox
{
	switch (td_iD) {
		case 1:
			if (uiDlgTag.dlgTags_cb_Tag1->currentText() !=  uiDlgTag.dlgTags_cb_Tag2->currentText() and uiDlgTag.dlgTags_cb_Tag1->currentText() !=  uiDlgTag.dlgTags_cb_Tag3->currentText() and uiDlgTag.dlgTags_cb_Tag1->currentText() !=  uiDlgTag.dlgTags_cb_Tag4->currentText()) {
				return 0;
			}
			break;
		case 2:
			if (uiDlgTag.dlgTags_cb_Tag2->currentText() !=  uiDlgTag.dlgTags_cb_Tag1->currentText() and uiDlgTag.dlgTags_cb_Tag2->currentText() !=  uiDlgTag.dlgTags_cb_Tag3->currentText() and uiDlgTag.dlgTags_cb_Tag2->currentText() !=  uiDlgTag.dlgTags_cb_Tag4->currentText()) {
				return 0;
			}
			break;
		case 3:
			if (uiDlgTag.dlgTags_cb_Tag3->currentText() !=  uiDlgTag.dlgTags_cb_Tag1->currentText() and uiDlgTag.dlgTags_cb_Tag3->currentText() !=  uiDlgTag.dlgTags_cb_Tag2->currentText() and uiDlgTag.dlgTags_cb_Tag3->currentText() !=  uiDlgTag.dlgTags_cb_Tag4->currentText()) {
				return 0;
			}
			break;
		case 4:
			if (uiDlgTag.dlgTags_cb_Tag4->currentText() !=  uiDlgTag.dlgTags_cb_Tag1->currentText() and uiDlgTag.dlgTags_cb_Tag4->currentText() !=  uiDlgTag.dlgTags_cb_Tag2->currentText() and uiDlgTag.dlgTags_cb_Tag4->currentText() !=  uiDlgTag.dlgTags_cb_Tag3->currentText()) {
				return 0;
			}
			break;
	}
	QMessageBox::information(this, tr("filmat11 information - tag already set !"), 
								   tr("The selected tag is already set as a search tag.\n"
									  "Clearing this one."));
	return 1;
}

void TagDlg::td_TagListAddTo()						// add new tag tag to list of tag tags
{
	td_slTags.append(td_sTagNew);
	td_slTags.sort();
}

int TagDlg::td_TagListIsIn()						// check if new tag tag already exists in list
{
	if (td_slTags.contains(td_sTagNew) == true) {
		return 0;
	}
	return 1;
}

void TagDlg::td_Tag1Chgd(int td_iD)					// tag 1 changed
{
	td_Tag1CbTextChgd();
}

void TagDlg::td_Tag2Chgd(int td_iD)					// tag 2 changed
{
	td_Tag2CbTextChgd();
}

void TagDlg::td_Tag3Chgd(int td_iD)					// tag 3 changed
{
	td_Tag3CbTextChgd();
}

void TagDlg::td_Tag4Chgd(int td_iD)					// tag 4 changed
{
	td_Tag4CbTextChgd();
}


// ****************************************************************
// **  tag dialog pop, handle changes							 **
// ****************************************************************
void TagDlg::td_Pop()								// populate edit movie tag dialog
{
	uiDlgTag.dlgTags_l_RootDir->setText(td_strMovieIn.sPath);
	uiDlgTag.dlgTags_l_FileName->setText(td_strMovieIn.sName);
	uiDlgTag.dlgTags_l_FileSuffix->setText(td_strMovieIn.sSuffix);
	uiDlgTag.dlgTags_l_FileSize->setText(QString::number(td_strMovieIn.fSize, 'f', 2));

	td_GenreCbCon(false);
	uiDlgTag.dlgTags_cb_Genre->setCurrentText(td_strMovieIn.sGenre);
	td_GenreCbCon(true);

	td_LangCbsCon(false);
	uiDlgTag.dlgTags_cb_Lang1->setCurrentText(td_strMovieIn.sLang1);
	uiDlgTag.dlgTags_cb_Lang2->setCurrentText(td_strMovieIn.sLang2);
	td_LangCbsCon(true);

	td_TagCbsCon(false);
	uiDlgTag.dlgTags_cb_Tag1->setCurrentText(td_strMovieIn.sTag1);
	uiDlgTag.dlgTags_cb_Tag2->setCurrentText(td_strMovieIn.sTag2);
	uiDlgTag.dlgTags_cb_Tag3->setCurrentText(td_strMovieIn.sTag3);
	uiDlgTag.dlgTags_cb_Tag4->setCurrentText(td_strMovieIn.sTag4);
	td_TagCbsCon(true);
}
