/*********************************************************************
 *  SPDX-License-Identifier: MIT									 *
 *  Copyright (C) 2010-2023 by Helge Kortendieck  alan@kasmithee.de  *
 *                                                                   *
 *  This file is part of filmat11.                                   *
 *********************************************************************/

#ifndef _TAGS_H_
#define _TAGS_H_

#include <QDialog>
#include <QFileDialog>
#include <QFileInfo>
#include <QLineEdit>
#include <QList>
#include <QMessageBox>
//#include <QSettings>
#include <QPoint>
#include <QSize>
#include <QString>
#include <QStringList>
#include <QWidget>

#include "rw.h"
#include "structs.h"

#include "ui_tags.h"

#include <iostream>		// debug only
using namespace std;	// debug only

class TagDlg : public QDialog		// edit tags dialog
{
	Q_OBJECT

	public:
		TagDlg(QDialog* parent = 0);
			~TagDlg();

	protected:

	private:
		bool td_bD;
//		bool td_bStrChgd;			// bool struct changed - true=yes update, false=no nothing to do
		bool td_bLiMoviesStrChgd;	// list with movies struct changed, requires re-load in filmat11

		int td_iCbChgdLang;			// changes in language combobox 1 or 2
		int td_iCbChgdTag;			// changes in language combobox 1 to 4
		int td_iCbChgdType;			// which tag type has been changed - 1=genre, 2=languages, 3=tags
		int td_iD;
		int td_iD2;
		int td_iD3;
		int td_iMovieCurIdx;		// index of current movie in list of movies
		int td_iRes;				// result value in function, will be returned
		int td_iRet;				// return value from function
		
		Ui::TagDlg uiDlgTag;				// edit tags dialog widget

		QDialog wDlgTag;					// edit tags dialog widget

		QLineEdit td_leD;

		QPoint qPos;

//		QSize size;

		QString td_sD;
		QString td_sD2;
		QString td_sD3;
		QString td_sD4;
		QString td_sTagNew;					// new tag text from editing the combobox
		QString td_sTagTxtCor;				// new corrected tag text - | replaced with _

		QStringList td_slD;
		QStringList td_slD2;
		QStringList td_slGenres;			// list of existing genres
		QStringList td_slLangs;				// list of existing languages
		QStringList td_slTags;				// list of existing tags

		StrMovie td_strMovieD;				// movie struct entity
		StrMovie td_strMovieIn;				// movie struct entity - status when called
		QList<StrMovie> td_liMoviesStrD;	// list with movie structs for movies table widget
		QList<StrMovie> td_liMoviesStr;		// list with movie structs for movies table widget

		StrTagsLists td_strTagsListsD;		// tags lists struct entity - 3 tag lists: genre, language and tags

	private slots:

		void td_CbsCon(bool td_bD);		// connect (true) / disconnect (false) comboboxes signals
		void td_CbsPop();				// populate comboboxes

		void td_ClickedClose();			// close clicked in edit tag dialog
		void td_ClickedDelTag();		// delete a Tag entry (genre, language or tag) clicked - show popup to select what to delete
		void td_ClickedSave();			// save clicked in edit tag dialog

		void td_Close();					// close edit tag dialog - reload movie str list, movie str list

		void td_GenreCbCon(bool td_bD);					// connect (true) / disconnect (false) genre combobox signals
		void td_GenreCbPop();							// populate genre combobox
		void td_GenreCbTextChgd();						// text in genre combobox has changed
		void td_GenreChgd(int td_iD);					// genre changed

		bool td_InputTxtChk(const QString &td_sD);		// check input text against | , replace any occurance with _ , true = changes where required

		void td_LangCbsCon(bool td_bD);					// connect (true) / disconnect (false) language combobox signals, no need to handle 1 and 2 separate ?
		void td_LangCbsPop();							// populate lang1+2 comboboxes
		void td_LangCbsPopRe();							// re-populate lang1+2 comboboxes
		void td_LangListAddTo();						// add new language tag to list of language tags
		int td_LangListIsIn();							// check if new language tag already exists in list
		void td_Lang1CbTextChgd();						// text in lang1 combobox has changed
		void td_Lang2CbTextChgd();						// text in lang2 combobox has changed
		int td_LangChkDbl();							// check language tag against double
		void td_Lang1Chgd(int td_iD);					// language 1 changed
		void td_Lang2Chgd(int td_iD);					// language 2 changed

		void td_ListsTagsGen();							// generate lists with tags - genres, languages and tags

		void td_Pop();									// populate edit movie tag dialog

		void td_SaveEnable(bool td_bD);					// save button: enable=true / disable=false save button

		void td_Show(QList<StrMovie> td_liMoviesStrD, int td_iD);	// show edit tag dialog

		void td_StrMovieChgd();							// movie struct has changed, set corresponding bool to true and enable save button

		void td_TagCbsCon(bool td_bD);					// connect (true) / disconnect (false) tags combobox signals, no need to handle 1 to 4 separate ?
		void td_TagCbsPop();							// populate tag1-4 comboboxes
		void td_TagCbsPopRe();							// re-populate tag 1-4 comboboxes
		void td_TagListAddTo();							// add new tag tag to list of tag tags
		int td_TagListIsIn();							// check if new tag tag already exists in list
		void td_Tag1CbTextChgd();						// text in tag1 combobox has changed
		void td_Tag2CbTextChgd();						// text in tag2 combobox has changed
		void td_Tag3CbTextChgd();						// text in tag3 combobox has changed
		void td_Tag4CbTextChgd();						// text in tag4 combobox has changed
		int td_TagChkDbl(int td_iD);					// check tag tag against double
		void td_Tag1Chgd(int td_iD);					// tag 1 changed
		void td_Tag2Chgd(int td_iD);					// tag 2 changed
		void td_Tag3Chgd(int td_iD);					// tag 3 changed
		void td_Tag4Chgd(int td_iD);					// tag 4 changed

/*		void settingsRead();
		void settingsWrite();
		void settingsWritePos();	// pos and size only
*/
		signals:
			void sigTagDlgClose(bool td_bD, QList<StrMovie> td_liMoviesStrD);	// close and delete edit tag dialog widget, true = re-read tags
};

#endif 
